* ------------------------------------------------------------------- SMC2FS2
      Program SMC2FS2


* Computes the Fourier amplitude spectra for specified time series.  
* Writes output in smc or column format.

* The output file name has the same name as the input file, with the specified
* character string appended to the name.

* Input from a control file; here is an example:
*!Control file for program SMC2FS2
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following 4 lines.
*! NOTE: Use prcntfrtaper with caution.  I recommend 0.0 at first.
*! I've been burned a few times
*! in computing FAS for unusual time series (eg., with long zero pads) in
*! which the FAS looked OK at first glance, but then I discovered that
*! it did not have the proper asymptotic properties or did not compare
*! properly with other FAS (in the zero pad case, for frequencies greater than
*! the lowest frequencies, the FAS for an unpadded and a 200s padded time series
*! were identical, but for a 600s pad, the FAS was noticeably smaller.  I finally 
*! tracked down the reason to the use of a 0.05% front taper.  For the long
*! time series for the 600s pad, the taper reduced signal).
*!
*! Meaning of smoothing input parameters
*!
*! NO SMOOTHING
*! itype = 0
*! SMOOTHING OVER EQUALLY SPACED FREQUENCIES
*! itype = 1: box weighting function
*!   smooth_param = width of box weighting function (Hz)
*! itype = 2: triangular weighting function
*!   smooth_param = width of triangular weighting function (Hz)
*! SMOOTHING OVER LOGARITHMICALLY SPACED FREQUENCIES
*! itype = 3: box weighting function
*!   smooth_param = xi, which is the fraction of a decade for the
*!                  box weighting function 
*! itype = 4: triangular weighting function
*!   smooth_param = xi, which is the fraction of a decade for the
*!                  triangular weighting function 
*! itype = 5: Konno and Ohmachi weighting function (see BSSA 88, 228-241)
*!   smooth_param = xi, which is the fraction of a decade spanned by the
*!                  first zero crossings for the
*!                  Konno and Ohmachi weighting function 
*!
*! ipow = power of FAS to be smoothed (2 = smoothing energy spectrum)
*!
*! df_smooth: Note: need df_smooth for linearly-spaced smoothers, 
*! and generally it should be the df from the fft.  The reason for
*! including it as an input parameter is to "fool" the
*! program to do smoothing over a specified number of points by
*! setting df = 1 and smooth_param = number of points (including 
*! points with zero weight at ends; e.g., smooth_param = 5 will 
*! give a smoother with weights 0, 1/4, 2/4, 1/4, 0; smooth_param
*! should be odd).
 !
*!Name of summary file:
* smc2fs2.sum
*pp: new set of parameters
*!tskip, tlength, prctnfrtaper, signnpw2(<0, backup for npw2, no zpad):
*   0.0 20.0 0.0 +1.0
*!dc_remove?
*  .true.        
*!smoothing: itype, ipow, df_smooth (0 = FFT df), smooth_param:
*  1  2 1.0 15.0
*!interpolation: df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high, log-spaced(0=F,1=T; integer)
*! NOTE: if log_spaced_f = T, then interpret df_intrp as the number of frequencies
*   0.0 0.0 50.0 0  
*!character string to append to filename:
*   .fs
*!Output in smc format (Y,N)?
*! ***IMPORTANT NOTE: Output cannot be in smc format if use log-spaced frequencies
*! because programs such as smc2asc have not been modified to deal with log-spaced
*! frequency.
* N
*!Files to process:
* \psv\taiwan99\t0781pea.smc
* \psv\taiwan99\t1293l3v.smc
* \psv\taiwan99\t0781ped.smc
*stop
 
* Dates: 06/15/01 - Written by D.M. Boore, patterned after SMC2FAS
*        06/27/01 - Replaced "_r" in f_fs2 with "_f".
*        07/10/01 - Write more stuff regarding processing parameters to 
*                   comments in file
*        07/23/01 - Changed filenaming convention.  Now I use the smc file,
*                   with the first two characters of the extension replaced
*                   by "fs".  I did this because otherwise duplicate filenames
*                   were being created.  For example, file.as1 and file.as2
*                   would have had the same output file names before 
*                   (file.fs2).   
*        10/19/01 - Obtain default name of control file via CR
*        11/11/01 - Write flow, df, fhigh, nfreq to comments
*        12/18/01 - Use double precision version of FORK (FORKDP) (had to
*                   modify absspect.for)
*        12/18/01 - Removed double precision version; it did not seem to 
*                   make a difference
*        12/18/01 - Store 0.0 if value is less than what can be written in the
*                   e format.
*        05/31/02 - More changes in file naming conventions.
*        07/17/03 - Add sps to smc_npts parameter list
*        12/08/03 - changed way of naming output (floowing blpadflt): tack on
*                   a specifed character string to the input file name.  This
*                   works because I now compile using LF95.
*        12/13/03 - Allow column file format as an option
*        12/29/03 - Add nfreq to column file output
*        10/29/04 - Modification so will reach desired fhigh
*        07/02/07 - Major change to structure of the control file, and include
*                   new, more generalized smoothing subroutine
*        07/03/07 - I deleted get path stuff
*        10/16/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        03/20/08 - Switch order of smoothing parameters on input.
*        03/20/08 - Add possibility of log-spaced frequencies.
*        03/24/08 - Write elapsed time for processing each record to a summary
*                   file.
*        03/24/08 - Add checks of f_intrp_high > fnyq
*        03/27/08 - Combine interpolation and smoothing to eliminate smoothing
*                   at unnecessary points.  This should speend up the Konno and
*                   Ohmachi smoother.
*        04/21/08 - Combined processing steps into a subroutine.
*        07/18/07 - Special purpose:  print complex spectral values for a few 
*                   frequencies, to check the relation between "signal" and
*                   "noise" spectra (this arose during Sinan Akkar's recent
*                   visit, where he added noise in the form of a baseline step
*                   to the TCU074 record; the FAS of the resulting signal
*                   was less than that of the FAS of the signal alone---I
*                   think that this is because there is an apparent
*                   correlation at low frequencies between signal and noise).
*                   To use, process one file per run, renaming
*                   "complex_fas.out" after each run (the output file is 
*                   written from within the subroutine).
*        12/19/08 - A new version, needed because smc2fs2 and its associated ctl file
*                   had changed enough that the previous version of smc2fs2_complex
*                   would not run.  Also, I increased the number of frequencies
*                   printed from within subroutine Abs_Spectra_Complex.  Because
*                   some processing steps were combined into Get_Abs_FAS, I had
*                   to make a new version (Get_Abs_FAS_complex) that calls Abs_Spectra_Complex. 

c
c Dimension and declaration statements:

      real fas(:), fas_out(:), freq(:), freq_out(:), ts(:)
      allocatable :: fas, fas_out, freq, freq_out, ts

      integer npts, nfreq_out, nc_f_smc
      real tskip, tlength, ftaper
      real df_intrp, f_intrp_low, f_intrp_high
      real df_smooth, signnpw2
      real t_smc, t_fft

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_smc*100, 
     :        f_fs2*100, f_sum*80, c2*2, c2up*2,
     :        ext_c*10, string*80, string4name*80

      character f_ctl*60, 
     :    date_begin*10, time_begin*11, time_start*11, time_stop*11

      logical f_ctl_exist, f_smc_exist, dc_remove, datetime_l
      logical smc_format, log_spaced_f

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2FS2.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2fs2.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2FS2:'
      date_begin = ' '
      time_begin = ' '
      call get_date(date_begin)
      call get_time(time_begin)
      write(nu_sum, '(2a)') ' Date: ', date_begin
      write(nu_sum, '(2a)') ' Time: ', time_begin
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
      if(buf_upper(1:3) .eq. 'PP:') then   ! a new set of processing parameters

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        read(nu_ctl,*) tskip, tlength_in, ftaper, signnpw2
        write(nu_sum,'(a)') 
     :     ' tskip, tlength_in, ftaper, signnpw2 ='
        write(nu_sum,*) 
     :     tskip, tlength_in, ftaper, signnpw2

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        read(nu_ctl,*) dc_remove
        write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        read(nu_ctl,*) itype, ipow, df_in, smooth_param
        write(nu_sum,'(a)') 
     :     ' itype, ipow, df_in, smooth_param ='
        write(nu_sum,*) 
     :     itype, ipow, df_in, smooth_param

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        read(nu_ctl,*) df_intrp, f_intrp_low, f_intrp_high, ilogspacef
        if (ilogspacef .eq. 0) then
          log_spaced_f = .false.
        else
          log_spaced_f = .true.
        end if
        write(nu_sum,'(a)') 
     :    ' df_intrp, f_intrp_low, f_intrp_high, log_spaced_f = '
        write(nu_sum, '(1x,1p, 3(1x,e10.3),0p, 1x,l1)') 
     :      df_intrp, f_intrp_low, f_intrp_high, log_spaced_f

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        string4name = ' '
        read(nu_ctl,'(a)') string4name
        call trim_c(string4name, nc_string4name)
        write(nu_sum,'(a)') 
     :   ' string4name = '//string4name(1:nc_string4name)

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
        buf = ' '
        read(nu_ctl,'(a)',end = 9999) buf
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:1)
        call upstr(buf_upper(1:1))
        if (buf_upper(1:1) .eq. 'Y') then
          smc_format = .true.
        else
          smc_format = .false.
        end if

        call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

        go to 1000
      end if

      f_smc = ' '
      f_smc = buf(1:nc_buf)
      call trim_c(f_smc, nc_f_smc)
      inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
      if (.not. f_smc_exist) then
          write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

* Extract the time series from the input file:

      print *
      print *, 
     :   'Processing file:', f_smc(1:nc_f_smc)
      write(nu_sum,*)
      write(nu_sum,'(1x,a)') 
     :   'Processing file:', f_smc(1:nc_f_smc)

      time_start = ' '
      call get_time(time_start)

* Get number of points on this loop:

      call smc_npts(f_smc(1:nc_f_smc), npts, sps)

      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

      n4_alloc = npts 
      allocate( ts(n4_alloc), fas_out(n4_alloc), freq_out(n4_alloc) )      
! Note: I should calculate the number of freq points, using the computations
! in get_abs_spectra.  For now, however, I assume that the number will always
! be less than npts (which will be greater than npw2/2, which is the number of
! FFT frequency points.

* Now read the data from the files:

      tlength = tlength_in
      call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               ts,  
     :               char_head, int_head, real_head, comments)
      sps = real_head(2)
      npts_smc = int_head(17)
      dt = 1.0/sps
      t_smc = float(npts_smc)*dt
      
      nacc_start = tskip * sps
      nacc_stop  = nacc_start + tlength * sps

      write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
     :     'signal_in: dt=', dt, 
     :     ' npts_smc=', npts_smc,' t_smc=', t_smc
      write(nu_sum,'(3x,a,1x,f8.2)') 
     :     ' tlength after smcread = ', tlength

* Compute the Fourier spectra:

      call Get_Abs_FAS_complex(
     : ts, sps, npts_smc, 
     : ftaper, signnpw2, dc_remove, nzpad, npw2, df_fft,  
     : itype, ipow, df_in, smooth_param, df_smooth,
     : df_intrp, f_intrp_low, f_intrp_high, log_spaced_f,
     : freq_out, fas_out, nfreq_out, df_out)
     
 
      deallocate( ts )

* Write output:

* Construct output filename:
  
      f_fs2 = ' '
      f_fs2 = f_smc(1:nc_f_smc)//
     :        string4name(1:nc_string4name)
      call trim_c(f_fs2,nc_f_fs2)

      print *, 
     :   'Writing output to file:', f_fs2(1:nc_f_fs2)
      write(nu_sum,'(2a)') ' Output file = ', f_fs2(1:nc_f_fs2)

      string = ' '
      string = '| Output of program SMC2FS2:'
      call trim_c(string, nc_string)
 
      datetime_l = .true.

      if (smc_format) then
        call FS2Write(f_fs2, nc_f_fs2, fas_out, 
     :              char_head, int_head, real_head, comments,
     :              f_smc, nc_f_smc, 
     :              nacc_start, nacc_stop, npw2,
     :              nfreq_out, freq_out(1), freq_out(nfreq_out), 
     :              df_out, df_smooth, ftaper,
     :              string,nc_string,datetime_l,
     :              tskip, tlength_in, df_intrp, f_intrp_low,
     :              f_intrp_high, signnpw2, dc_remove, nzpad)
      else
* Open output file:
        call get_lun(nu_fs2)
        open(unit=nu_fs2, file=f_fs2(1:nc_f_fs2), status='unknown')

* Write column heads:
        write(nu_fs2, '(a)') ' Output of SMC2FS2: FAS for file '//
     :                       f_smc(1:nc_f_smc)
        write(nu_fs2, '(a)') ' Nfreq_out = '
        write(nu_fs2, '(1x,i6)') nfreq_out
        write(nu_fs2,'(8x,a, 9x,a)') 'freq', 'fas'
      
        do j = 1, nfreq_out
          write(nu_fs2,'(1p,2(1x,e11.4))') 
     :       freq_out(j), fas_out(j)
        end do

        close(nu_fs2)
      end if

      deallocate (fas_out, freq_out)

      time_stop = ' '
      call get_time(time_stop)

      call time_diff(time_start, time_stop, time_elapsed)
      write(nu_sum, '(2x,a,1x,f6.1)') 
     :       ' Elapsed time (sec): ', time_elapsed
      write(nu_sum,*)
      write(nu_sum,*)

      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2FS2


      include '\forprogs\get_abs_fas_complex.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\locate.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\fs2write.for'
      include '\forprogs\timediff.for'
      include '\forprogs\get_date.for'
      include '\forprogs\get_time.for'

 