		      Program BLPadFlt
		      
! This is a major reorganization of Bl_Fltr that takes advantage of the
! longer file names allowed in Fortran 90 and is intended for batch
! processing of many files using the same input parameters.  In the future I
! might insert "pp:" lines as in BlFlt2.For to allow changing some of the 
! input parameters for groups of files.

! See the comment lines at the beginning of Bl_Fltr for various warnings 
! and comments


!!Control file for program BLPadFlt
!!
!! Note on use of "!" to denote a comment line.  This is only used
!! for lines in the group of input lines containing processing
!! parameters.  Once "pp:" is encountered, the program assumes that
!! all following lines until the next "pp:" or "stop" contain
!! the names of files to be processed (note that this is consistent
!! with file name restrictions:  starting a file name with "!" is legal).
!!
!! Revision of program involving a change in the control file on this date:
!   07/11/17
!!Summary file name:
! blpadflt.sum
!!
!! NOTE: Those parameters most likely to stay fixed are immediately below,
!! before the list of files.  Those likely to change for various groups
!! of files are set below the "pp:" line.
!!
!!PARAMETERS FOR OBTAINING THE MEAN:
!!
!! mean info (contained after "pp:" line below):
!! type of mean (0=no correction, 1=standard average; -1=specify mean)
!! t4mean_strt, t4mean_stop, mean (use if type = -1)
!! NOTE: need entries even if the parameter is not used (because read
!! info use list-directed [unformatted] input) 
!!
!!PARAMETER FOR DETRENDING AFTER MEAN REMOVAL STEP:
!!
!! 0 = NO; 1 = YES  (the detrending removes a straight line fit to the first
!!                   and last points)
!!
!!PARAMETERS FOR BASELINE ADJUSTMENT:
!!
!! Info is after "pp:" line below; here are the meanings:
!! -Option (0=spcfy T1 & T2; 1=Iwan 1; 2=Iwan 2; 3=t1,t2 from velfit; 10= no bl)
!!   (option 4: remove pulse between T1 and T2, whose area equals the mean of 
!!    the velocity from t2b to t2e)
!! -Fit d or v (0, 1), norder (1:line;2:quad.;3:cubic, etc-- 11 max), prcntfrtpr
!! -Array indicating which coefficients are fixed at 0 (0) or are free (1);
!!  there must be norder+1 values
!! -Torigin, T1b, T1e, T2b, T2e: The equation fitted to the velocity or 
!!  displacement is relative to an origin at t = Torigin.  For example, 
!!  a straightline fit to velocity will have the equation 
!!  v = c(1) + c(2)*(t-torigin).  The Torigin option was included to allow for 
!!  a baseline correction starting at t1=t2=torigin,
!!  in particular when the polynomial is constrained to be 0.0 at t=torigin 
!!  (this is specified by setting the first element in the array of coefficients
!!  in the preceeding line to 0.0). The coefficients of the fitted polynomial 
!!  are chosen so as to minimize the sum of the squared residuals between 
!!  T1b--T1e and T2b--T2e.  The first range (T1b--T1e) was included following 
!!  the suggestion of V. Graizer to allow a portion near the beginning of the 
!!  record to have some influence on the fitted function; in practice, this 
!!  option is rarely used, and the usual specification is T1b = T1e = 0.0.
!!
!!PARAMETERS FOR ZERO PADDING:
!!
!! Add pads if apply acausal filter (Y/N)?  This should almost always be
!! set to "Y".   Even if it is set to "N", note that the subroutine that
!! does the filtering adds pads (but it does not go to the first zero crossing 
!! and it does not save the padded time series)
! Y
!! Factor by which the number of zeros is increased beyond that determined by
!! the standard (BAP) equation (using a number greater than 1.0 may be necessary if the
!! full effect of the filter at very low frequencies, as seen in plots of FAS,
!! is to be obtained; see dependence_of_fourier_amplitude_spectra_at_low_frequencies_on_zero_pads.pdf
!! in http://www.daveboore.com/daves_notes.html for an example and discussion).
! 1.0
!! Replace discarded portions of original data with zeros (Y/N)?  By doing this,
!! two horizontal components with the same number of time points before
!! padding (the usual case) will have the same number after padding and can
!! be rotated.
! Y
!!
!! Go to first zero crossing?
! Y
!! If go to first zero crossing = Y, use first point rather than the zero crossing 
!! if smaller than first zero crossing (this parameter is ignored if specify "N" above,
!! in which case the first point is always used)?
! Y
!! Go to last zero crossing?
! Y
!! If go to last zero crossing = Y, use last point rather than the zero crossing 
!! if smaller than last zero crossing (this parameter is ignored if specify "N" above,
!! in which case the last point is always used)?
! Y
!!
!! IMPORTANT NOTE: Care must be taken if the zero pads start at a point 
!! determined by going inward to the first or last zero crossings
!! ("Go to first/last zero crossing = Y"). Look at the headers of the smc files
!! to see how many data points were skipped.
!! Sometimes the beginning or end of the unfiltered time series has a drift, in
!! which case too many points will be skipped before the first zero crossing is
!! reached.
!!
!!    11/03/06 - If zcross_l or zcross_t = F and use_first or use_last = T, 
!!               pad with values equal to the last points.  Allow this option
!!               to deal with cases such as filtering a displacement
!!               time series with a low-frequency drift, which will have
!!               a step offset if pad with zeros at the end.  Be careful in
!!               in using this option for the first points, however (in
!!               a future version I may eliminate the option for the beginning,
!!               as generally the leading pad should be zero).  The need for
!!               this option arose when I tried to filter a displacement
!!               obtained by integrating a zeroth-order corrected acceleration.
!!               The results were different than filtering the acceleration
!!               before integration.
!!               IF WANT TO PAD WITH ZEROS, STARTING AT 
!!               BEGINNING AND END OF DATA, SET ZCROSS_L, ZCROSS_T = F AND
!!               USE_FIRST, USE_LAST = F (This is potentially confusing, and
!!               I may remove the option of padding with the first and last
!!               values in a later version of the program)
!!
!! Remove mean before determining segment to be padded?
!! If Yes, the mean will be determined from the whole record.  If this
!! is not desireable, remove the mean outside of this program
! N
!! Remove mean after determining segment to be padded (the segment
!! will probably be different than the segment originally used to determine the
!! mean, and therefore the mean will no longer be 0.0. But then after removing
!! this mean, the segment to be padded will not be at 0.0 values, but I 
!! hope this is a small problem.  I probably should iterate):
! N
!! Save padded time series?  (".pad" will be appended to file name)
! Y
!!
!!PARAMETERS FOR FILTERING:
!!
!! flc (0 = no filter), nslope_lc, icaus_lc (1=causal),
!! fhc (0 = no filter), nslope_hc, icaus_hc (1=causal)
!!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
!!   enforced in the program)
!! note that nslope is the asymptotic power of frequency, so that the filter
!! goes as f^{nslope} for a low-cut filter and f^{-nslope} for a 
!! high-cut filter. In previous versions I used "nroll" as the parameter 
!! controlling the asymptotic behavior of the filter, the reason being that it 
!! is the parameter used by the filter routines in BAP.   This was always
!! confusing, so I switched to nslope on 27 Oct. 2005, with a translation to
!! nroll in blpadflt (nroll = 0.5*nslope for causal and 
!! nroll = 0.25*nslope for acausal). (Note more confusing terminology:
!! nroll = 0.5*npoles for a causal Butterworth filter.) 
!! The response of an acausal filter
!! is not that of a Butterworth, because time domain filters are used.
!! Bazzuro et al. (2004 COSMOS paper) use the terminology npnp, where
!! "n" is the number of poles of a standard Butterworth filter.  The "npnp"
!! notation is used to indicate that two passes of a Butterworth filter are
!! used.   So, for example, "2p2p" means that two passes of a two-pole
!! Butterworth filter are used, with an asymptotic dependence as f^4 (for a
!! low-cut filter). This notation is used to distinguish this case
!! from that in which an acausal filter is simulated in the frequency domain, in
!! which case the filter can have a true Butterworth response function. For that 
!! case, a "4p" acausal filter constructed in the frequency 
!! domain and using the standard Butterworth response would have the same
!! asymptotic dependence (f^4).
!!
!! I added a notch filter on 03/20/10.  The notch filter is invoked if
!! (flc = fhc) and flc /= 0.0, and the width parameter "d" of the filter is given by 
!! d = real(nslope_lc)*10.0**real(-abs(nslope_hc)).  The filter will be causal if 
!! both icaus_lc = 1 and icaus_hc = 1. Adding this new filter did
!! not require a change in the input parameters, but I changed the date of the
!! control file to make the user aware of a change in the capability of the
!! program. Here is a sample specification:
!!   25.0 1 1 25.0 2  1
!! which corresponds to a 25 Hz causal notch filter with filter width of 
!! 1*10**(-2)= 0.01, which is the starting trial value recommended by 
!! Bill Joyner, who wrote the notch filter subroutine.
!!
!! On 07/11/17 I allowed for the low-cut and high-cut to be causal or acausal
!! separately; previously, both were either causal or acausal.  I made the change
!! because causal low-cut filters can affect high frequency response spectra, whereas
!! there might be some advantages to using a causal high-cut filter because the filter
!! is down by 1/sqrt 2 at the corner frequency rather than 1/2 (thus giving a sharper transition
!! between the unfiltered and filtered portions of the spectra).  Previously I need to run blpadflt
!! twice to achieve this, the first time with an acausal low-cut filter, and then using
!! the resulting padded time series in a second run of blpadflt for a causal high-cut
!! filter.  
!!
!! In the previous version, the filter routine "band" was called if both flc and fhc /= 0.0 and
!! flc /= fhc.  The possibility of different types of filters for flc and fhc means that I need to call
!! locut and hicut separately, but I want to retain the possibility of a band filter.  For that reason I
!! added a Y/N parameter to indicate if a band filter is desired. If Yes, then it will be
!! a causal filter only if both icaus_lc and icaus_hc = 1.
!!
!!ACCURACY OF TIME SERIES OUTPUT 
!!Use the higher precision format for the output time series files,
!!even if the input file has the standard precision? (Y/N) 
! Y
!!
!!PARAMETERS FOR RESPONSE SPECTRA:
!!
!!Periods for response spectra (nperiods log spaced from per_low to per_high):
! 200 0.01 90
!!Damping values (ndamp, damp(1), damp(2), ..., damp(ndamp)):
! 1 0.05 0.20
!!Sinc interpolate factor (IF) (this MUST be a power of two; e.g., 2, 4, 8, etc.;
!! IF = 1 returns the original data):
!! sps_out = interpolate_factor* sps_in
!! IF = 8 is high enough, because the sinc interpolation assumes that there
!! is no energy beyond the Nyquist data of the original data.  This is 0.5*sps_in.
!! If want to resolve an oscillator response with this frequency using at least 10 points
!! per period, this means resampling so that sps_in = 10 * 0.5*sps_in = 5 *sps_in.
!! Because a power of 2 is needed, IF = 8 is sufficient (see comparisons in Wang et al, PEER 2018/xx)
!! Note that the resampling of the filtered time series is used only for computing response spectra;
!! the smc output files are at the same sps as the input.  If resampled smc files are desired,
!! use the program smc_interpolate_time_series_using_fork.
! 8
!!Use straightline interpolation if Tosc < 10*dt (even after any sinc interpolation)
! Y
!!
!!PROCESSING PARAMETERS AND FILES, IN GROUPS
!!
!pp: new set of parameters
!!tskip tlength  
! 0.0 400.0
!!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
!!mean (use if type = -1):
! 1  0.0 8.0  -999.
!!detrend after mean removal (0=no, 1=yes, in which case a line fit to
!!the first and last points is removed; note that the mean need not be removed
!!before detrending if type = 0 above.  In addition, caution should be used in
!!using this option for "normal" time series; it is intended as a quick
!!baseline correction for the infrequent cases in which there is a large
!!linear trend in the time series):
! 0
!!Option (0=spcfy T1 & T2; 1=Iwan 1; 2=Iwan 2; 3=t1,t2 from velfit; 4=pulse; 
!!        10= no bl)
! 10
!!T1,T2 (options 0,4;A1,A2 if <0);A1,A2 (option 1); T1 (option 2; A1 if T1<0.0):
! 10.0 20.0 
!!Fit d or v (0, 1), norder (1:line;2:quad.;3:cubic, etc-- 11 max), prcntfrtpr:
! 1  3  0
!!Array showing coefficients fixed at 0 (0) or free (1); must be norder+1 
!!values:
! 0 0 1 1   ! cubic, with first and second terms constrained to zero
!!Torigin, T1b, T1e, T2b, T2e (fit between T1b--T1e and T2b--T2e):
! 0.0  0.0 0.0 56.0 200.0
!!
!!Filter information (see earlier text for information):
!! flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc
!! (flc, fhc = 0 for no filter; icaus = 0 for acausal, 1 for causal)
!! (nslope >= 2: nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal)
! 0.02 8  0 0.0  4  0
!! If this a band filter (Y/N)? (the entry is not used if either flc or fhc = 0 or if
!! flc == fhc)
! N
!!
!!Length of taper at beginning and end of time series, before adding zeros
!! (Not used if no filter or a causal filter, but still need this parameter
!! (in the input file):
! 0.0 20.0
!!character string to append to filename to make the output filename:
!! if this character string equals "<auto>", the extension will be constructed
!! from the filter information (e.g., for a 0.125 Hz acausal low-cut filter 
!! with a slope of 8, the extension will be '.alc00.125ns08').
! .alc0p02_taper_20s
!!integrate to velocity and displacement? (y/n):
! y
!!Are the units of acceleration in g (Y/N); if N, I assume that the units are
!! cm/s/s.  If something else, it is up to the user to scale the output appropriately 
!! (the output units are as follows: sd(cm); pv(cm/s); pa(cm/s/s); rv(cm/s); aa(cm/s/s); ag (g)
!! where the output columns indicate the type of response spectra, as follows:
!! sd=relative displacement
!! pv=pseudo relative velocity
!! pa=pseudo absolute acceleration (aka PSA)
!! rv=relative velocity
!! aa=absolute acceleration
!! ag=pseudo absolute acceleration, in g.
!! (see aa_pa_rv_pv_2.pdf on the Dave's informal notes on a variety of topics page of www.daveboore.com)
! N
!!Names of files to be processed with the parameters above:
!501053xa.raw.smc
!pp: new set of parameters
!!tskip tlength  
! 0.0 400.0
!!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
!!mean (use if type = -1):
! 1  0.0 8.0  -999.
!!detrend after mean removal (0=no, 1=yes, in which case a line fit to
!!the first and last points is removed; note that the mean need not be removed
!!before detrending if type = 0 above):
! 0
!!Option (0=spcfy T1 & T2; 1=Iwan 1; 2=Iwan 2; 3=t1,t2 from velfit; 4=pulse; 
!!        10= no bl)
! 10
!!T1,T2 (options 0,4;A1,A2 if <0);A1,A2 (option 1); T1 (option 2; A1 if T1<0.0):
! 10.0 20.0 
!!Fit d or v (0, 1), norder (1:line;2:quad.;3:cubic, etc-- 11 max), prcntfrtpr:
! 1  3  0
!!Array showing coefficients fixed at 0 (0) or free (1); must be norder+1 values:
! 0 0 1 1   ! cubic, with first and second terms constrained to zero
!!Torigin, T1b, T1e, T2b, T2e (fit between T1b--T1e and T2b--T2e):
! 0.0  0.0 0.0 56.0 200.0
!!
!!Filter information (see earlier text for information):
!! flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc
!! (flc, fhc = 0 for no filter; icaus = 0 for acausal, 1 for causal)
!! (nslope >= 2: nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal)
! 0.04 8 0 0.0  4  1
!! If this a bandpass filter (Y/N)? (the entry is not used if either flc or fhc = 0 or if
!! flc == fhc)
! N
!!
!!Length of taper at beginning and end of time series, before adding zeros
!! (Not used if no filter or a causal filter, but still need this parameter
!! (in the input file):
! 0.0 20.0
!!character string to append to filename to make the output filename:
!! if this character string equals "<auto>", the extension will be constructed
!! from the filter information (e.g., for a 0.125 Hz acausal low-cut filter 
!! with a slope of 8, the extension will be '.alc00.125ns08').
! <auto>
!!integrate to velocity and displacement? (y/n):
! y
!!Are the units of acceleration in g (Y/N); if N, I assume that the units are
!! cm/s/s.  If something else, it is up to the user to scale the output appropriately 
!! (the output units are as follows: sd(cm); pv(cm/s); pa(cm/s/s); rv(cm/s); aa(cm/s/s); ag (g)
!! where the output columns indicate the type of response spectra, as follows:
!! sd=relative displacement
!! pv=pseudo relative velocity
!! pa=pseudo absolute acceleration (aka PSA)
!! rv=relative velocity
!! aa=absolute acceleration
!! ag=pseudo absolute acceleration, in g.
!! (see aa_pa_rv_pv_2.pdf on the Dave's informal notes on a variety of topics page of www.daveboore.com)
! N
!!Names of files to be processed with the parameters above:
!501053xa.raw.smc
!stop
  
! Dates: 11/26/03 - Written by D. Boore, based on Bl_Fltr and SMC_Pad
!        11/28/03 - Include check for existence of f_in and 
!                   whether t2b < tend
!        12/25/03 - Change writing of time series after removing pads to
!                   include enough of the zeros so that the input and output
!                   time series are the same length and there is no apparent
!                   time shift as a result of going to the first zero crossing.
!        06/01/04 - Add option to control file so that pads need not be added
!                   if apply acausal filter (for experimenting with processing,
!                   there may be cases where want to do a series of filters
!                   on a specified time series).
!        06/03/04 - Fix an error in which the time series was shifted to the 
!                   left by one point.
!        06/04/04 - Added option to replace discarded portions of original
!                   time series with zeros.
!        10/27/05 - Use "nslope" rather than "nroll" to specify filters; add
!                   "version_ctl" to control file to prevent using old version of
!                   control file if the control file has been changed in the
!                   program.  Usually the control does not change, but it 
!                   did for today's revision.  BE SURE TO RESET VERSION_CTL
!                   IF NECESSARY (only if alter the control file)
!        03/13/06 - Remove f_in from smcpadf calling parameter list
!        11/03/06 - Modified smcpadf to trick it into using the first or last
!                   values of the unpadded time series for the pad values
!                   (see comments in the sample control file above for
!                   the reason for doing so).
!        01/24/07 - Revised smcpadf, and thus used a new version date here
!                   and in blpadflt.ctl
!        01/24/07 - Added option to taper end of time series before adding trailing
!                   zero pad.
!        07/15/07 - Added option to taper start of time series before adding leading
!                   zero pad.
!        10/16/07 - Allow for higher precision smc files; the output files will
!                   have the same precision as the input files.
!        02/25/08 - Allow for longer input and output smc filenames
!        05/24/08 - Trap for t_zero < 0.0 or > tend for option 3, in which case
!                   quit processing the record and skip to the next record.
!                   In addition, allow the option 3 time and offset to be determined
!                   from a fit to displacement or to velocity (previously, a fit to
!                   velocity was imposed).
!        06/19/08 - Modification neede to correct bug when i_stem+3 is greater than
!                   12, the character size of "header".
!        09/13/08 - Correct error: nslope_bp not assigned for causal bandpass filter.
!        10/23/08 - Add indices used in computing mean to comments in output file
!        10/26/08 - Include options for increasing the number of leading and trailing
!                        zeros by a multiplicative factor.  This option was included
!                        because of some results that indicated that the FAS of a low-cut 
!                        filtered time series needed more filter transients to capture
!                        the effect of filtering at low frequencies.
!        10/28/08 - Include option for integrating to velocity and displacement (if "n",
!                   then assume that the time series is of unknown type).
!        02/06/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
!                   values when processing a list of files.  The problem is that if a file with
!                   a shorter record length is processed before one with a longer record
!                   length, a mean supposedly determined for the whole record was determined
!                   only for the length of the shorter preceeding record.
!        02/15/09 - Add option to detrend after the mean removal step (of course, the mean
!                   need not be removed).
!        05/22/09 - Include option for constructing the filename extension from the filter
!                   information.
!        05/29/09 - Using a blank in the control file to indicate the automatic construction
!                   of the file name bothered me (I kept wanting to delete the blank line)
!                   so I have changed it to "<auto>".
!        06/03/09 - Replaced n_t4mean with dble(n_t4mean) in defining the mean avg_all.
!        06/25/09 - Use get_date and get_time, which call the standard Fortran 90 DATE_AND_TIME
!                   subroutine rather than the compiler-specific subprograms date and time.
!        10/19/09 - If file to be processed is not found, write message to summary file.
!        10/19/09 - Modified trim_c so that it first replaces tabs with spaces, before
!                   removing leading and trailing spaces.  I did this to correct a subtle
!                   bug, revealed when I was processing a long list of files.   For some
!                   files the program insisted that the file did not exist, when it clearly
!                   did exist in the working folder.  I had Textpad show spaces and tabs,
!                   and as a result I discovered that the "missing" files had a tab after
!                   their name in the control file.  Deleting the tab corrected the problem.
!                   I modified trim_c to automate this. 
!        12/08/09 - Add front and back taper info to character string use to create the 
!                   output file name created if "<auto>" is specified for the character string
!                   in the control file.
!        01/25/10 - Correct error in construct_filename_extension.for when auto naming the 
!                   output file if only remove mean (no filtering)
!        03/06/10 - Substituted RSCALC_INTERP_ACC for RDRVAA.  RSCALC_INTERP_ACC interpolates the
!                   acceleration time series to smaller dt if period < 10.0*dt, but uses rdrvaa
!                   otherwise.
!        03/10/10 - Added a notch filter; see above for description of how to specify its parameters.
!        03/21/10 - Add frequency to response spectrum output.
!                 - Add "a" to output filename when no velocity or time series are computed (this
!                   corrects a bug).
!        12/04/10 - Include tskip. tlength as processing parameters (useful, for example, if have three components 
!                   at a site and want to exclude an aftershock).
!        12/21/10 - Store indices of first and last pad-stripped value in int_head(31,32) (for acausal filters)
!        12/27/10 - Add option for output time series to be stored using higher precision smc format even
!                   if the input file has standard precision (if the input file has higher
!                   precision, the output format will also be higher precision).
!        06/13/11 - Get rid of some of the go to and statement numbers by using DO, CYCLE, EXIT, using 
!                   smc2rs_ts.for as an example.
!        11/18/12 - Changed description of filter order requirement from "power of 2" to "factor of 2"
!        11/22/16 - Do not include the path of the input file in the output file name.
!        07/11/17 - Allow the low- and high-cut filters to be either causal or acausal, independently
!                   (previously they had to both be one or the other).
!                 - Because of the possibly different type of low- and high-cut filters, I no longer assume
!                   a band pass filter if flc and fhc > 0.0 (in practice, flc and fch are often much different,
!                   so using the locut followed by the hicut filters makes sense, rather than the band filter).
!        07/13/17 - Use filter_type and reset icaus, nslope variables only once.
!        02/22/18 - Allow icaus to be different for lc and hc filters.
!                 - A major revision, allowing sinc interpolation in the program (based on 
!                   smc_interpolate_time_series_using_fork.for).  This is only used in the computation
!                   of response spectra
!                 - Ask if units of accel are g.
!        02/24/18 - Move setting of nslope_bp, nroll_bp before checks of acausal, causal for a band filter
!        02/27/18 - Correct error in assigning skip_straightline_interp (it
!                   was set to T when the input from the control file asking
!                   if linear interpolation was to be used was set to Y). The logical 
!                   way to do this is to change skip_straightline_interp to use_straightline_interp.
!                   This requires changes to a number of routines that use rs_calc, but
!                   is the right thing to do.
!        03/11/18 - Write notch d parameter to real_head(35)
!        06/26/19 - Add more smc file comments, correct error insmc file comment that says used 
!                   acausal filter when it was a causal filter.
!        01/09/20 - Cleanup writing of smc text header when no filter is used, and also delete the erroneous
!                   "QUITTING" message when filter type = NF
!                 - Change portion of automatic filename for baseline correction from 
!                   "blc" to "v0_blc" for option 3. (This change was made in construct_filename_extension.for.)

!                   

 
 

      external dfpoly

      parameter (npc = 12)   ! max order of polynomial plus 1
      parameter (small = 0.01) 

      character char_head(11)*80, comments(500)*80, 
     :                            cmnts2skip(100)*79
     
      character date_ctl_correct*8, date_ctl_in*30
     
      integer int_head(48), option, interpolate_factor
      
      integer nc, ic_in(npc)
      real*8 c(npc),covar(npc,npc)  ! allows up to npc-1 order polynomial fit
      real*8 pd(3)            ! allows up to 2 derivatives of the poly fit
      real*8 chisq, t_dp
      real real_head(50)

      real a_in(:), a_bl(:), a_out(:), v_out(:), d_out(:), work(:)
      real y_out(:), time_r(:)
      allocatable :: a_in, a_bl, a_out, v_out, d_out, 
     :               y_out, time_r, work

      real*8 x(:), y(:), sig(:)
      allocatable :: x, y, sig

      double precision avg
      character f_ctl*60, f_sum*60, f_in*300, buf*300, buf_c*79,
     :          path*300
      character f_stem*300,
     :          f_a*300, f_v*300, f_d*300, f_rs*300
      character string_to_append*200, buf3*3, buf4*4, buf6*6, f_pad*300
      character compute_v_d*10, precision*8, high_precision4output*8
      character filter_type*10

      logical f_ctl_exist, f_in_exist, rmv_trnd, add_impulse
      logical zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after, detrend
      logical apply_pads, save_padded_time_series, 
     :        replace_discarded_data_with_zeros,
     :        use_straightline_interp, accel_units_g,
     :        band_filter
     
      integer read_status
      
      real rd(5), rv(5), aa(5), damp(5), period(1000)
      real omega, omega2
      character header(27)*12, damp_c*2
      character date_c*10, time_c*11

      pi = 4.0*atan(1.)
      twopi = 2.0*pi

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("Enter" = blpadflt.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'blpadflt.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

! Check date of control file:
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/22/18'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:
      
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(a)') ' Summary file for program BLPadFlt'
      write(nu_sum,'(a)') 
     :     '   Using control file: '//f_ctl(1:nc_f_ctl)


! PARAMETERS FOR ZERO PADDING:

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        apply_pads = .true.
      else
        apply_pads = .false.
      end if

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) xfactor_zpads

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        replace_discarded_data_with_zeros = .true.
      else
        replace_discarded_data_with_zeros = .false.
      end if

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_l = .true.
      else
        zcross_l = .false.
      end if

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_first = .true.
      else
        use_first = .false.
      end if

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_t = .true.
      else
        zcross_t = .false.
      end if

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_last = .true.
      else
        use_last = .false.
      end if

      write(*,'(a)') ' zcross_l, zcross_t = '
      write(*, *)      zcross_l, zcross_t

      write(*,'(a)') ' use_first, use_last = '
      write(*, *)      use_first, use_last

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_before = .true.
      else
        remove_mean_before = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_before = ', remove_mean_before

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_after = .true.
      else
        remove_mean_after = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_after = ', remove_mean_after

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        save_padded_time_series = .true.
      else
        save_padded_time_series = .false.
      end if

!PRECISION OF SMC OUTPUT TIME SERIES FILES
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      do ihc = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(ihc)
      end do
      high_precision4output = ' '
      read(nu_ctl,'(a)') high_precision4output
      call trim_c(high_precision4output, nc_high_precision4output)
      call upstr(high_precision4output(1:nc_high_precision4output))
      write(nu_sum,'(2x,a)') 
     :           high_precision4output(1:nc_high_precision4output)

! PARAMETERS FOR RESPONSE SPECTRA:

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      do ihc = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(ihc)
      end do

      read(nu_ctl,*) nperiods, per_low, per_high
      write(nu_sum,*) nperiods, per_low, per_high

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      do ihc = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(ihc)
      end do
      read(nu_ctl,*) ndamp, (damp(i), i = 1, ndamp)
      write(nu_sum,*) ndamp, (damp(i), i = 1, ndamp)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   interpolate_factor  ! This must be a power of two--check
      if (modulo(log(real(interpolate_factor)), log(2.0)) /= 0) then
        print *,' interpolate_factor= ', 
     :              interpolate_factor, ' not a power of 2; QUIT!!'
        close(nu_ctl)
        stop
      end if

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf_c)
      if (buf_c(1:1) == 'Y') then
        use_straightline_interp = .true.
      else
        use_straightline_interp = .false.
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)

! CHECK FOR ANOTHER FILE TO PROCESS

      loop over input lines: DO

        write(nu_sum,*)  
        
        buf = ' '
        read(nu_ctl,'(a)',iostat = read_status) buf
        if (read_status /= 0) then
          EXIT loop over input lines
        end if
        write(nu_sum,'(1x,a)') buf
        call trim_c(buf,nc_buf)
        
        buf3 = ' '
        buf3 = buf(1:3)
        call upstr(buf3)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        
        IF (buf4 == 'STOP') EXIT loop over input lines
        
        IF (buf3 == 'PP:') THEN
        
! New processing parameters

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          read(nu_ctl,*) tskip, tlength      
          write(nu_sum,'(1x,a,2(1x,es10.3))') 
     :         ' tskip, tlength=', tskip, tlength
      
! PARAMETERS FOR OBTAINING THE MEAN:

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          read(nu_ctl,*) itype_mean, 
     :                 t4mean_strt_in, t4mean_stop_in, spec_mean
          write(nu_sum,*)  itype_mean, 
     :                   t4mean_strt_in, t4mean_stop_in, spec_mean

! PARAMETER FOR DETRENDING:

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          read(nu_ctl,*) i_detrend
          if (i_detrend == 0) then
            detrend = .false.
          else
            detrend = .true.
          end if

! PARAMETERS FOR BASELINE ADJUSTMENT:

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          read(nu_ctl,*) option
          write(nu_sum,*) option

! If option = 10, skip through the specification of parameters for fitting
! a baseline:

          IF (option == 10) THEN
          
            DO i = 1, 4
              call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
              call skip(nu_ctl,1)
            END DO
            
          ELSE  

            call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
            do ihc = 1, nc_cmnts2skip
              write(nu_sum,'(1x,a)') cmnts2skip(ihc)
            end do
            if (option == 0) then
              read(nu_ctl,*) t1, t2
              write(nu_sum,*) t1, t2
            else if (option == 1) then
              read(nu_ctl,*) a1, a2
              write(nu_sum,*) a1, a2
            else if (option == 2) then
              read(nu_ctl,*) t1
              write(nu_sum,*) t1
            else if (option == 3) then
            call skip(nu_ctl,1)
            write(nu_sum,'(a)') 
     :     ' Because option = 3, t1, t2 are calculated in the program'
            else if (option == 4) then
              read(nu_ctl,*) t1, t2
              write(nu_sum,*) t1, t2
            else  ! error.... option not used
              write(nu_sum,'(a,1x,i3,a)') '  !!! ERROR, option= ',
     :        option, ' is not a recognized value; QUITTING !!!'
              write(*,'(a,1x,i3,a)') '  !!! ERROR, option= ',
     :        option, ' is not a recognized value; QUITTING !!!'
              close(nu_ctl)
              close(nu_sum)
              stop
            end if

            call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
            do ihc = 1, nc_cmnts2skip
              write(nu_sum,'(1x,a)') cmnts2skip(ihc)
            end do
            read(nu_ctl,*) idvfit, iorder, prcntfrtpr
            if (idvfit /= 0 .and. idvfit /= 1) then
              write(nu_sum,'(a)') 
     :                   ' ERROR!!! idvfit <> 0, <> 1; QUITTING!!!)'
              write(*,'(a)') ' ERROR!!! idvfit <> 0, <> 1; QUITTING!!!)'
              close(nu_ctl)
              close(nu_sum)
              stop
            end if          

            if (iorder == 0) then
              write(nu_sum,'(a)') ' ERROR!!! iorder = 0; QUITTING!!!)'
              write(*,'(a)') ' ERROR!!! iorder = 0; QUITTING!!!)'
              close(nu_ctl)
              close(nu_sum)
              stop
            end if          

            write(nu_sum,*) idvfit, iorder, prcntfrtpr
          
            call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
            do ihc = 1, nc_cmnts2skip
              write(nu_sum,'(1x,a)') cmnts2skip(ihc)
            end do
            read(nu_ctl,*) (ic_in(i),i=1,iorder+1)

            if (option == 3) then   ! v0 case, force all parameters to be free
              do i = 1, iorder+1
                ic_in(i) = 1
              end do
              write(nu_sum,'(a)') ' Option = 3, so set all ic_in = 1'
            end if
            write(nu_sum,*) (ic_in(i),i=1,iorder+1)
        
            call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
            do ihc = 1, nc_cmnts2skip
              write(nu_sum,'(1x,a)') cmnts2skip(ihc)
            end do
            read(nu_ctl,*) torigin, t1b, t1e, t2b, t2e
            write(nu_sum,*) torigin, t1b, t1e, t2b, t2e

          END IF   ! Get baseline parameters


! PARAMETERS FOR FILTERING:

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          do ihc = 1, nc_cmnts2skip
            write(nu_sum,'(1x,a)') cmnts2skip(ihc)
          end do
          read(nu_ctl,*) flc, nslope_lc, icaus_lc,
     :                   fhc, nslope_hc, icaus_hc
          write(nu_sum,'(a)') ' Filter parameters upon input:'
          write(nu_sum,*) flc, nslope_lc, icaus_lc,
     :                    fhc, nslope_hc, icaus_hc

!Check for bandpand filter:
          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          buf = ' '
          read(nu_ctl,'(a)') buf
          call upstr(buf)
          call trim_c(buf, nc_buf)
          do ihc = 1, nc_cmnts2skip
            write(nu_sum,'(1x,a)') cmnts2skip(ihc)
          end do
          write(nu_sum,'(a)') buf(1:nc_buf)
          if (buf(1:1) == 'Y') then
            band_filter = .true.
          else
            band_filter = .false.
          end if
          
! Determine filter type and rest parameters if needed, including
! enforcing a factor of two and four for causal and acausal filters, respectively

! First assign filter type:
          filter_type = ' '
          if (flc == 0.0 .and. fhc == 0.0) then
            filter_type = 'NF'
            band_filter = .false.
          else if (fhc == 0.0 .and. flc > 0.0) then
            filter_type = 'LC'
          else if (flc == 0.0 .and. fhc > 0.0) then
            filter_type = 'HC'
          else if (flc == fhc ) then  ! notch
            filter_type = 'NOTCH'
          else if (band_filter) then
            filter_type = 'BAND'
          else
            filter_type = 'LCHC'
          end if

          call trim_c(filter_type,nc_filter_type)
          
          write(nu_sum,*)  
          write(nu_sum,'(a)') ' filter type = '//
     :                          filter_type(1:nc_filter_type)
          write(nu_sum,*)  

! Set some parameters if a filter is used:

          if (filter_type(1:nc_filter_type) /= 'NF') then
          
! Set icaus_bn and reset icaus_lc, icaus_hc for icaus_bn = 0:

            if (filter_type(1:nc_filter_type) == 'NOTCH' .or.
     :        filter_type(1:nc_filter_type) == 'BAND') then 
              if(icaus_lc == 1 .and. icaus_hc == 1) then
                icaus_bn = 1
              else
                icaus_bn = 0
                icaus_lc = 0  
                icaus_hc = 0
! for a notch or bandpass filter, it needs to be either causal or acausal; 
! the proper way to specify this for an acausal filter is to set icaus_lc = icaus_hc = 0
! in the control file, but sometimes one is set to 0 and the other to 1.  For that
! reason I have reset them to 0 for an acausal filter
              end if
            else
              icaus_bn = 0
            end if

! Reset nslope if needed:
            if (icaus_lc == 0) then   ! acausal lc  
              if (nslope_lc < 4) nslope_lc = 4
              nslope_lc = 4 * int(float(nslope_lc)/4.0)
              nroll_lc = nslope_lc/4
            else                      ! causal lc
              if (nslope_lc < 2) nslope_lc = 2
              nslope_lc = 2 * int(float(nslope_lc)/2.0)
              nroll_lc = nslope_lc/2
            end if
            if (icaus_hc == 0) then   ! acausal hc  
              if (nslope_hc < 4) nslope_hc = 4
              nslope_hc = 4 * int(float(nslope_hc)/4.0)
              nroll_hc = nslope_hc/4
            else                      ! causal hc
              if (nslope_hc < 2) nslope_hc = 2
              nslope_hc = 2 * int(float(nslope_hc)/2.0)
              nroll_hc = nslope_hc/2
            end if

! Set notch parameter:
            if (filter_type(1:nc_filter_type) == 'NOTCH') then
              d4notch = real(nslope_lc)*10.0**(-abs(real(nslope_hc)))
            else
              d4notch = 0.0
            end if
          
            write(nu_sum,*)  
            write(nu_sum,'(a)') ' Filter parameters after enforcing'//
     :       ' factor of two and four for causal and acausal filters:'
            write(nu_sum,'(a)') ' filter type = '//
     :                          filter_type(1:nc_filter_type)
            write(nu_sum,'(a)') ' flc, nslope_lc, icaus_lc,'// 
     :                        ' fhc, nslope_hc, icaus_hc, icaus_bn'
            write(nu_sum,*) flc, nslope_lc, icaus_lc,
     :                    fhc, nslope_hc, icaus_hc, icaus_bn
          
            write(nu_sum,'(a)') ' and nroll_lc, nroll_hc'
            write(nu_sum,*) nroll_lc, nroll_hc
            write(nu_sum,'(a)') ' and d4notch'
            write(nu_sum,*) d4notch
            write(nu_sum,*) 
            
          end if ! set some filter parameters

! OPTION TO TAPER END OF TIME SERIES BEFORE ADDING LEADING AND
! TRAILING ZERO PADS

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          do ihc = 1, nc_cmnts2skip
            write(nu_sum,'(1x,a)') cmnts2skip(ihc)
          end do
          read(nu_ctl,*) taper_beginning, taper_end
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
          write(nu_sum,*)  
 
! GET STRING TO APPEND TO FILE NAME:

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          string_to_append = ' '
          read(nu_ctl,'(a)') string_to_append
          call trim_c(string_to_append, nc_s2appnd)
          buf6 = ' '
          buf6 = string_to_append(1:nc_s2appnd)
          call upstr(buf6)
          
          if (buf6(1:6) == '<AUTO>') then ! construct extension
            call construct_filename_extension
     :      (tskip, tlength,
     :       option, flc, nslope_lc, icaus_lc,
     :               fhc, nslope_hc, icaus_hc, icaus_bn,
     :               filter_type, nc_filter_type,
     :       taper_beginning, taper_end, 
     :       string_to_append, nc_s2appnd)
          end if
          

          write(nu_sum,'(a)') ' string_to_append: '//
     :          string_to_append(1:nc_s2appnd)
          write(nu_sum,*)  

! Integrate to v and d?

          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
          compute_v_d = ' '
          read(nu_ctl,'(a)') compute_v_d
          call trim_c(compute_v_d, nc_compute_v_d)
          call upstr(compute_v_d(1:1))

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call upstr(buf_c)
          call trim_c(buf_c,nc_buf_c)
          if (buf_c(1:1) == 'Y') then
            accel_units_g = .true.
          else
            accel_units_g = .false.
          end if
      
          call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)

          CYCLE loop over input lines
          
        END IF  ! end loop over "PP:"


! Read the input file:

        f_in = ' '
        call trim_c(buf, nc_buf)
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)

        f_in_exist = .false.
        
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          write(nu_sum,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          CYCLE loop over input lines
        end if

        write(*,'(a)') ' Processing file: '//f_in(1:nc_f_in)
        write(nu_sum,*)  
        write(nu_sum,'(a)') ' Processing file: '//f_in(1:nc_f_in)

        call smc_npts(f_in(1:nc_f_in), npts_max, sps)
        allocate (a_in(npts_max), a_bl(npts_max), 
     :            x(npts_max), y(npts_max), sig(npts_max), 
     :            time_r(npts_max)) 

!        tskip = 0.0
!        tlength = -1.0

        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               a_in, char_head, int_head, real_head, comments)
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        npts = int_head(17)
        tend = float(npts-1)/sps

        write(nu_sum,'(a)') ' sps, ncomments, npts = '
        write(nu_sum, *) sps, ncomments, npts

! Scale to cm/s/s if in g:
        if (accel_units_g) then
          g2gals = 981.0
          do i = 1, npts
            a_in(i) = g2gals * a_in(i)
          end do
        end if
        
! Write comment headers (NOTE: I am distributing these throughout
! rather than in one place as in Bl_Fltr because smcpadf writes
! comment within the subroutine, and I wanted the comments in proper order.)

        call get_date(date_c)
        call get_time(time_c)

        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '| Processing input file '//f_in(1:nc_f_in)//
     :      ' using program BlPadFlt'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,es10.3)') 
     :  '|    tskip = ', tskip
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,es10.3)') 
     :  '|    tlength = ', tlength
     
! Need to allocate a_out, v_out, d_out differently because
! more than npts_max might be used in acausal filter routines (those
! routines pad the array beyond npts).  Must do this even if choose not to
! add pads when do an acausal filter.

! NOTE: the pad computation here is done because of the need to allocate the
! time series; nlz and ntz are passed into subroutine smcpadf.

! It is more efficient, but not by much, to use a series of if then else statements.  This is
! more readable, even though each statement must be evaluated.


!In a previous version there is a problem for bandpass filter
! with different npads for 
! acausal when icaus_lc or hc = 1; check to see if this is still true.

! NOTE: npad = 0 for all causal filters (careful for locut followed by hicut)

        npad = 0  ! only change if acausal
        npad_lc = 0  ! only change if acausal
        npad_hc = 0  ! only change if acausal
        if (filter_type(1:nc_filter_type) /= 'NF') then 
          if (filter_type(1:nc_filter_type) == 'LC') then
            if (icaus_lc == 0) then ! acausal locut only
              npad = 3.0*float(nroll_lc)/(flc*dt)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'HC') then
            if (icaus_hc == 0) then ! acausal hicut only
              npad = 3.0*float(nroll_hc)/(fhc*dt)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'NOTCH') then 
            if (icaus_bn == 0) then ! acausal notch
              npad = 3.0*pi/d4notch
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'BAND') then 
            nslope_bp = max0(nslope_lc, nslope_hc)
            nroll_bp = max0(nroll_lc, nroll_hc)                ! make sure that these were defined earlier as if they are acausal
            if (icaus_bn == 0) then ! acausal bandpass
              npad1=3.0*float(nroll_lc)/(flc*dt)
              npad2=6.0*float(nroll_bp)/((fhc-flc)*dt)
              npad = max0(npad1, npad2)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'LCHC') then
              if (icaus_lc == 0) then
                npad_lc = 3.0*float(nroll_lc)/(flc*dt)
              else
                npad_lc = 0
              end if
              if (icaus_hc == 0) then
                npad_hc = 3.0*float(nroll_hc)/(fhc*dt)
              else
                npad_hc = 0
              end if
              npad = npad_lc + npad_hc
          else
             write(*,'(a)') 
     :           ' In npad part of blpadflt, unknown filter type ('//
     :             filter_type(1:nc_filter_type)//'); quitting!!! '
             stop
           end if
         end if

         write(nu_sum,'(a)') 
     :  ' For filter type = '//filter_type(1:nc_filter_type)//
     :  ', flc, icaus_lc, fhc, icaus_hc, icaus_bn, npad_lc,'//
     :  '  npad_hc, npad:' 
        write(nu_sum,*) 
     :    flc, icaus_lc, fhc, icaus_hc, icaus_bn, npad_lc, npad_hc, 
     :    npad
        write(nu_sum,*) 


! moved this out of IF loop
        if (xfactor_zpads < 1.0) then
          npad4alloc =  npad  ! need this because the allocations in
                              ! subroutine locut assume that npad is at
                              ! least as large as the standard value.
        else
          npad4alloc = int(xfactor_zpads * float(npad))
        end if
        npad = int(xfactor_zpads * float(npad))

        nlz = npad/2
        ntz = npad/2
        izcl = nlz
        izct = ntz

        npts4alloc = npts_max + 2*npad4alloc + 2
        
!        npts4alloc = npts_max + nlz + ntz + npad + 2

        write(nu_sum,'(a)') ' After determination of npad: '//
     :     'npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc ='
        write(nu_sum,*) 
     :      npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc

! extra trailing pad of npad because the filter subroutines add their own 
! zero pad equal to npad to the end of the time series.  If no integration was
! being done, then that would be OK, but I want to add npad/2 to the front so 
! that the integration accounts properly for the filter transient. 
! I first added only an extra npad/2, but this led to a hard-to-find run time
! error.

        allocate (a_out(npts4alloc))
        allocate (v_out(npts4alloc))
        allocate (d_out(npts4alloc))

        do i = 1, npts4alloc
          a_out(i) = 0.0
          v_out(i) = 0.0
          d_out(i) = 0.0
        end do

! Step 1: REMOVE MEAN FROM WHOLE RECORD

!  Determine the appropriate mean:

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
        if (itype_mean == 0) then
          n_t4mean = 0
          avg_all = 0.0
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   no mean removed.'
        else if (itype_mean == -1) then   ! use specified mean
          avg_all = spec_mean
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|   remove the specified mean ', avg_all,
     :      ' from whole record.'
        else if (itype_mean == 1) then
          if (t4mean_strt < 0.0) t4mean_strt = 0.0
          if (t4mean_stop > tend) t4mean_stop = tend
          n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
          n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
          n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
          t4mean = float(n_t4mean -1)/sps
          avg = 0.0d0
          do i = n_t4mean_strt, n_t4mean_stop
            avg = avg + dble(a_in(i))
          end do
          avg_all = sngl(avg/dble(n_t4mean))
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|    remove the computed mean = ', avg_all,
     :        ' from whole record;'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1p,e12.5,a,e12.5)') 
     :    '|    mean determined from average between t= ', 
     :     t4mean_strt, ' and t = ', t4mean_stop
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|    corresponding to n_t4mean_strt = ', n_t4mean_strt
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|                     n_t4mean_stop = ', n_t4mean_stop
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|                      and n_t4mean = ', n_t4mean
        else            ! shouldn't be here!
          write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          write(*) '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          EXIT loop over input lines
        end if

!  Remove the mean from the whole record:
        write(nu_sum,'(a,1x,i6)') ' n_t4mean = ', n_t4mean
        write(nu_sum,'(a,1x,1pe12.5,1x,a)') 
     :   ' Removing mean = ', avg_all, ' from the whole record'

        do i = 1, npts
          a_out(i) = a_in(i) - avg_all
          time_r(i) = float(i-1)*dt
        end do

        
! Step 2: DETREND

        if (detrend) then
          call rmvtrend(a_out, npts)
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   detrend the time series by subtracting '//
     :    'a line fit to the first & last points' 
        end if	 

! Step 3: BASELINE ADJUSTMENT

        if (option == 10) then  ! remove overall baseline only
          d0bl = 0.0
          v0bl = 0.0
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          ncomments = ncomments + 1
          comments(ncomments) = 
     :         '| No baseline adjustment (option = 10)'
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          go to 7777   
        end if

!   Integrate to velocity and displacement

        rmv_trnd = .false.
  
        call acc2vd(a_out, npts, dt, rmv_trnd, 0.0, 0.0, 
     :              v_out, d_out)
        v_last = v_out(npts)
        d_last = d_out(npts)
  
        write(nu_sum,'(a,1p2(1x,e11.4))') 
     :    ' v_last, d_last = ', v_last, d_last

!   Polynomial fit to parts of disp. or vel. time series

        if (option == 4) then  ! remove pulse from t1 to t2, whose area equals the average
                                 ! velocity from t2b to t2e.
! Option 4 gets special treatment:

          if (t2b >= tend) then
            write(*,'(a,f6.1,a,f6.1,a)')
     :       ' ** t2b = ', t2b, ', > tend = ', tend,
     :       ', moving to next file **'
            go to 6245
          end if

          n_t2b = nint(sps * t2b) + 1
          if (t2e > tend) t2e = tend
          n_t2e = nint(sps * t2e) + 1
          t2b = float(n_t2b-1)/sps
          t2e = float(n_t2e-1)/sps
          n_seg2 = n_t2e - n_t2b + 1 
          avg = 0.0d0
          do i = 1, n_seg2
            avg = avg + dble(v_out(i))
          end do
          avg_vel = sngl(avg/dfloat(n_seg2))
          goto 4444
        end if

!   First extract the portion to fit:

! first work with beginning section (if there is one):
        n_seg1 = 0
        if (t1e <= 0.0) then
          n_t1b = 0
          n_t1e = 0
          ntaper1 = 0
        else
          n_t1b = nint(sps * t1b) + 1
          if (t1e > tend) t1e = tend
          n_t1e = nint(sps * t1e) + 1
          t1b = float(n_t1b-1)/sps
          t1e = float(n_t1e-1)/sps
          n_seg1 = n_t1e - n_t1b + 1 
          ntaper1 = float(n_seg1)  * prcntfrtpr/100.0
          if (idvfit == 0) then      ! fit displacement
            do i = 1, n_seg1
              x(i) = time_r(i + n_t1b - 1) - torigin    ! time origin = torigin
              y(i) = wind_box(i,1,n_seg1,ntaper1) * d_out(i + n_t1b - 1)
              sig(i) = 1.0
            end do
          else if (idvfit == 1) then  ! fit velocity
            do i = 1, n_seg1
              x(i) = time_r(i + n_t1b - 1) - torigin    ! time origin = torigin
              y(i) = wind_box(i,1,n_seg1,ntaper1) * v_out(i + n_t1b - 1)
              sig(i) = 1.0
            end do
          end if
        end if

! next work with second section (if there is one):
        n_seg2 = 0
        if (t2e <= 0.0) then
          n_t2b = 0
          n_t2e = 0
          ntaper2 = 0
        else
          if (t2b >= tend) then
            write(*,'(a,f6.1,a,f6.1,a)')
     :       ' ** t2b = ', t2b, ', > tend = ', tend,
     :       ', moving to next file **'
            go to 6245
          end if
          n_t2b = nint(sps * t2b) + 1
          if (t2e > tend) t2e = tend
          n_t2e = nint(sps * t2e) + 1
          t2b = float(n_t2b-1)/sps
          t2e = float(n_t2e-1)/sps
          n_seg2 = n_t2e - n_t2b + 1 
          ntaper2 = float(n_seg2)  * prcntfrtpr/100.0
          if (idvfit == 0) then      ! fit displacement
            do i = 1, n_seg2
              x(i+n_seg1) = time_r(i + n_t2b - 1) - torigin !time orgn = torigin
              y(i+n_seg1) = wind_box(i,1,n_seg2,ntaper2) * 
     :                      d_out(i + n_t2b - 1)
              sig(i+n_seg1) = 1.0
            end do
          else if (idvfit == 1) then  ! fit velocity
            do i = 1, n_seg2
              x(i+n_seg1) = time_r(i + n_t2b - 1) - torigin !time orgn = torigin
              y(i+n_seg1) = wind_box(i,1,n_seg2,ntaper2) * 
     :                      v_out(i + n_t2b - 1)
              sig(i+n_seg1) = 1.0
            end do
          end if
        end if
          
        n_tfit = n_seg2 + n_seg1

!   Now fit the curve:
     
        nc = iabs(iorder) + 1

        do i = 1, npc
          c(i)     = 0.0
        end do                

        call dlfit(x, y, sig, n_tfit, c, ic_in, nc,
     :     covar, npc, chisq, dfpoly)

        if (idvfit == 0) then
          write(nu_sum,'(a, i2, a)') 
     :     ' Fit order ', iorder, ' polynomial to displacement: '
          write(nu_sum,'(a)') 
     :     '   torigin, prcntfrtpr, '//
     :     't1b, t1e, n_seg1, ntaper1, '//
     :     't2b, t2e, n_seg2, ntaper2, '//
     :     'n_tfit = '
          n_v = 2  ! used in dddpoly to specify which term is velocity
        else if (idvfit == 1) then
          write(nu_sum,'(a, i2, a)') 
     :     ' Fit order ', iorder, ' polynomial to velocity: '
          write(nu_sum,'(a)') 
     :     '   torigin, prcntfrtpr, '//
     :     't1b, t1e, n_seg1, ntaper1, '//
     :     't2b, t2e, n_seg2, ntaper2, '//
     :     'n_tfit = '
          n_v = 1  ! used in dddpoly to specify which term is velocity
        else
          write(*,'(a)') ' ERROR!!! idvfit <> 0 or 1; QUITTING!!!)'
          stop
        end if
        write(nu_sum,'(2(1x,f8.3), 
     :       2(1x,f8.3), 2(1x,i5),
     :       2(1x,f8.3), 2(1x,i5),
     :       1x,i5)') 
     :       torigin, prcntfrtpr, 
     :       t1b, t1e, n_seg1, ntaper1,
     :       t2b, t2e, n_seg2, ntaper2,
     :       n_tfit
       
        write(nu_sum,'(t17,a)') 
     :        ' and: coefficients = '
        write(nu_sum,'(1p12(1x,e12.5))') (c(i),i=1, nc)

!    term in dddpoly that will yield acceleration
!    (first index is the variable fitted, either velocity or displacement)
        n_a = n_v + 1

4444    continue

!   Determine t1, t2:

        if (option == 0) then
          if(t1 < 0.0) then      ! use amp
            a1 = abs(t1)
            call a1_to_t1(a_out, npts, dt, a1, t1)
            if (t1 < 0.0) EXIT loop over input lines  ! all amps < a1; quitting.
          end if
          if(t2 < 0.0) then      ! use amp
            a2 = abs(t2)
            call a2_to_t2(a_out, npts, dt, a2, t2)
            if (t2 < 0.0) EXIT loop over input lines  ! all amps < a2; quitting.
          end if
        else if (option == 1) then
          call a1_to_t1(a_out, npts, dt, a1, t1)
          if (t1 < 0.0) EXIT loop over input lines  ! all amps < a1; quitting.
          call a2_to_t2(a_out, npts, dt, a2, t2)
          if (t2 < 0.0) EXIT loop over input lines  ! all amps < a2; quitting.
        else if (option == 2) then
          if(t1 < 0.0) then      ! use amp
            a1 = abs(t1)
            call a1_to_t1(a_out, npts, dt, a1, t1)
            if (t1 < 0.0) EXIT loop over input lines  ! all amps < a1; quitting.
          end if
! Determine t2 by minimizing the final displacement.  Do this by
! brute force, rather than by using brent or golden.
          n1 = sps * t1 + 1
          t2 = t1

! ----------- TO BE CHANGED ----------------------------
! Evaluate fitted polynomial and its derivatives
          t_dp = 0.0-torigin
          call dddpoly(c, nc, t_dp, pd, n_a)
          v0bl = pd(n_v)
          af = pd(n_a)
! REDO THIS AND D_BASELINE, USING MORE GENERAL EXPRESSIONS AND ALLOWING
! FOR A FIT WHOSE ORIGIN IS AT TORIGIN.  IF FIT
! VELOCITY, THEN MUST "INTEGRATE" TO DISPLACEMENT
          d_resid_min = abs(d_last - d_baseline(t1,t2,tend,v0bl,af))
! ----------- TO BE CHANGED ----------------------------

          t2_min = t2
          do i = n1+1, npts
            t2 = float(i-1)/sps   ! consider evaluating at larger time spacing
            d_resid = abs(d_last - d_baseline(t1,t2,tend,v0bl,af))
            if( d_resid < d_resid_min) then
              d_resid_min = d_resid
              t2_min = t2
            end if              
          end do
          t2 = t2_min
          write(nu_sum,'(a,1p2(1x,e11.4))')
     :     ' d_resid_min, t2_min = ', d_resid_min, t2_min
        else if (option == 3) then
          if (idvfit == 0) then  ! fit displacement
            t1 = torigin - 0.5*c(2)/c(3)
          else if (idvfit == 1) then
            t1 = torigin - c(1)/c(2)
          else
            print *,' ***ERROR***, idvfit = ', 
     :            idvfit, 
     :              ' /= 0 or 1; moving to next file.'
            write(nu_sum,'(a,1x,i4,1x,a)') ' ***ERROR***, idvfit = ',
     :            idvfit, 
     :            ' /= 0 or 1; moving to next file.'
            go to 6245
          end if
          if (t1 < 0.0 .or. t1 > tend) then
            print *, ' ***ERROR***, t1 = ', t1, 
     :           ' outside range of time series, moving to next file.'  
            write(nu_sum,'(a,1x,1p,e10.3,1x,a)')
     :           ' ***ERROR***, t1 = ', t1, 
     :           ' outside range of time series, moving to next file.' 
            go to 6245
          end if
          t2 = t1
        else if (option == 4) then
          dum = 0.
! Nothing needs to be done
        else if (option > 4 .and. option /= 10) then ! should not be here!
          null = 0
        end if

        n1 = nint(sps * t1) + 1
        n2 = nint(sps * t2) + 1
        t1 = float(n1-1)/sps
        t2 = float(n2-1)/sps
        write(nu_sum,'(a,2(1x,i5),2(1x,f8.3))') 
     :     '  n1, n2, t1, t2 = ', n1, n2, t1, t2

!   Determine and subtract acceleration baseline:

! Initialize the baseline:
        do i = 1, npts
          a_bl(i) = 0.0
        end do

! Set initial displacement and velocity
        d0bl = 0.0
        v0bl = 0.0

        if (option == 4) then
          am = avg_vel/(t2 - t1)
          write(nu_sum,'(a,1x,1pe12.5,0p2(a,1x,f8.3))') 
     :     '  subtracting am = ', am, 
     :          ' between t1 = ', t1, 
     :              ' and t2 = ', t2 
          do i = n1, n2-1
            a_bl(i) = am
          end do
          go to 5555
        end if

! Fill out a_bl from n2 to end:
        do i = n2, npts
          t_dp = time_r(i)-torigin
          call dddpoly(c, nc, t_dp, pd, n_a)
          a_bl(i) = pd(n_a)
        end do

! Figure out what to do between t1 and t2:
        add_impulse = .false.
        t_dp = (t1-torigin)
        call dddpoly(c, nc, t_dp, pd, n_a)
        v1t1 = pd(n_v)
        if (n2 /= n1) then   ! Case when n1 <> n2:
          t_dp = t2-torigin
          call dddpoly(c, nc, t_dp, pd, n_a)
          v2 = pd(n_v)
          am = v2/(t2 - t1)
          write(nu_sum,'(a,1x,1pe12.5,0p2(a,1x,f8.3))') 
     :     '  subtracting am = ', am, 
     :          ' between t1 = ', t1, 
     :              ' and t2 = ', t2 

          do i = n1, n2-1
            a_bl(i) = am
          end do
        else if (t1 == 0.0 .and. t2 == 0.0) then
!  use real_header to store v0, but do not put an impulse at t1
          t_dp = (0.0-torigin)
          call dddpoly(c, nc, t_dp, pd, n_a)
          v0bl = pd(n_v)
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :       '  t1 = t2 = 0.0, v0bl, a_bl(t1) = ', v0bl, a_bl(n1)
        else if ( abs(v1t1) > small) then
!  n1 = n2, not equal to 0.0, and v at t1 not equal to 0.0,
!  implies a step in velocity, so put an impulse at t1 equal to v1/dt
          t_dp = (t1-torigin)
          call dddpoly(c, nc, t_dp, pd, n_a)
          v1 = pd(n_v)
          a_bl(n1) = v1/dt + a_bl(n1)  ! so gives correct answer if v1 = 0.0
          write(nu_sum,'(a,1p3(1x,e12.5))') 
     :     '  t1 = t2 <> 0.0, add impulse: t1, t2, v1, a_bl(t1) = ', 
     :          time_r(n1), time_r(n2), v1, a_bl(n1)
          add_impulse = .true.
        else
          t_dp = (t2-torigin)
          call dddpoly(c, nc, t_dp, pd, n_a)
          v1 = pd(n_v)
          write(nu_sum,'(a,1p3(1x,e12.5))') 
     :     '  t1 = t2 <> 0.0, no impulse: t1, t2, v1, a_bl(t1) = ', 
     :          time_r(n1), time_r(n2), v1, a_bl(n1)
        end if

        if (idvfit == 1) then
          write(nu_sum,'(2(a,1x,f8.3))') 
     :     '  subtracting single derivative of fitted velocity'//
     :      ' between t2 = ', t2, 
     :      ' and tend = ', time_r(npts) 
        else
            write(nu_sum,'(2(a,1x,f8.3))') 
     :     '  subtracting double derivative of fitted displacement'//
     :      ' between t2 = ', t2, 
     :      ' and tend = ', time_r(npts) 
        end if

5555    continue

        do i = 1, npts
          a_out(i) = a_out(i) - a_bl(i)
        end do
        npts_out = npts

        if (option == 10) then        ! remove overall baseline only
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          ncomments = ncomments + 1
          comments(ncomments) = 
     :         '| No baseline adjustment (option = 10)'
        else                           ! other options
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i1)') 
     :    '|  t2 determined using option ', option
          if (n2 /= n1) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x,1pe12.5,0p2(a,1x,f8.3))') 
     :        '|  subtracting am = ', am, 
     :              ' between t1 = ', t1, 
     :                  ' and t2 = ', t2 
          else if (t1 == 0.0 .and. t2 == 0.0) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x,1pe12.5)') 
     :        '|  subtracting baseline from t = 0.0, with v0bl = ', 
     :         v0bl 
          else if (add_impulse) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a)') 
     :        '|  t1 = t2 <> 0.0, step in velocity, add impulse at t1'
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1p3(1x,e12.5))') 
     :       '|     where t1, v1, a_bl(t1) = ', 
     :                        time_r(n1), v1, a_bl(n1)
          else if (option == 3) then
            if (idvfit == 0) then
              ncomments = ncomments + 1
              write(comments(ncomments),'(a)') 
     :          '|  option 3: t1 = t2 from zero crossing of velocity' 
              ncomments = ncomments + 1
              write(comments(ncomments),'(a)') 
     :          '|  derived from a quadratic fit to displacement'
            else
              ncomments = ncomments + 1
              write(comments(ncomments),'(a)') 
     :          '|  option 3: t1 = t2 from zero crossing of velocity' 
              ncomments = ncomments + 1
              write(comments(ncomments),'(a)') 
     :          '|  derived from a linear fit to velocity'
            end if            
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x,1pe12.5,a,1x,e12.5)') 
     :        '|    from t1b = ', t1b, ' to t1e = ', t1e
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x,1pe12.5,a,1x,e12.5)') 
     :        '|    from t2b = ', t2b, ' to t2e = ', t2e
            ncomments = ncomments + 1
            write(comments(ncomments),'(a)') 
     :        '|  t1 = t2, > torigin, no added impulse at t1'
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1p3(1x,e12.5))') 
     :       '|     where torigin, t1, a_bl(t1) = ', 
     :        torigin, time_r(n1), a_bl(n1)
          else
            ncomments = ncomments + 1
            write(comments(ncomments),'(a)') 
     :        '|  t1 = t2 <> 0.0, no step in velocity, '//
     :           'no added impulse at t1'
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1p3(1x,e12.5))') 
     :       '|     where t1, v1, a_bl(t1) = ', 
     :                         time_r(n1), v1, a_bl(n1)
          end if

          if (option == 4) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x, 1p, 1x, e10.3)') 
     :        '|  option 4: avg_vel = ', avg_vel
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1x,1pe12.5,a,1x,e12.5)') 
     :        '|    from t2b = ', t2b, ' to t2e = ', t2e
            go to 8765
          end if
          
          if( idvfit == 0) then
            ncomments = ncomments + 1
            comments(ncomments) = 
     :        '|  baseline determined by fitting displacement'
          else
            ncomments = ncomments + 1
            comments(ncomments) = 
     :        '|  baseline determined by fitting velocity'
          end if
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2)') 
     :        '|   with a polynomial of order ', iabs(iorder)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,12(1x,i1))') 
     :        '|   with array ic_in (0=coefficient fixed at 0.0) = ', 
     :        (ic_in(i), i = 1, nc)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3,a,f8.3,a)') 
     :        '|   fit between ', t1b, ' and ', t1e, ' sec and'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3,a,f8.3,a)') 
     :        '|   fit between ', t2b, ' and ', t2e, ' sec'//
     :        ' with regression coefficients: '
          ncomments = ncomments + 1
          nc_up = min0(6, nc)
          write(comments(ncomments),'(a,1p,6(1x,e12.5))') 
     :        '| ', (c(i),i=1, nc_up)
          if (nc > 6) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1p,6(1x,e12.5))') 
     :        '| ', (c(i),i=nc_up+1, nc)
          end if
8765      continue
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          int_head(16) = ncomments
        end if

7777    continue

! Set initial velocity, displacement

        d0 = - d0bl
        v0 = - v0bl ! minus to account for subtraction of baseline

! Step 4: PAD TIME SERIES IF USE ACAUSAL FILTERS

        if (npad /= 0) then ! acausal filter, apply pads

          write(nu_sum,'(a)') 
     :          ' Inside pad time series if acausal filter:'
          write(nu_sum,'(a)') ' Working on file '//f_in(1:nc_f_in)
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f6.2)') 
     :            '| Acausal filter, apply pads, '//
     :            'increasing the number of zeros by '
  
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f6.2)') 
     :            '|   the multiplicative factor ', xfactor_zpads
 
          call smcpadf(a_out, npts, sps, nlz, ntz, 
     :      replace_discarded_data_with_zeros, 
     :      taper_beginning, taper_end, 
     :      npts_out, izcl, izct,  
     :      remove_mean_before, remove_mean_after, 
     :      zcross_l, use_first, zcross_t, use_last,
     :      comments, ncomments)    

! Store new ncomments, npts:

          int_head(16) = ncomments 
          

        else

          nlz = 0
          ntz = 0
          izcl = 1
          izct = npts
          npts_out = npts

        end if

! Step 5: FILTER

        if (flc /= 0.0 .or. fhc /= 0.0) then

          write(nu_sum, '(a,1x,i7)') 
     :          ' Before call to filter, npts_out = ', npts_out

          call filter(a_out, npts_out, dt, flc, nroll_lc, icaus_lc,
     :              fhc, nroll_hc, icaus_hc, icaus_bn,
     :              d4notch, filter_type, nc_filter_type)
        end if

! Step 6: INTEGRATE TO VELOCITY AND DISPLACEMENT

        if (compute_v_d == 'Y') then
  
          v04vd = 0.0
          d04vd = 0.0

          rmv_trnd = .false.
          call acc2vd(a_out, npts_out, dt, rmv_trnd, v04vd, d04vd, 
     :              v_out, d_out)
        end if

! Step 7: WRITE THE NEW SMC FILE (AND IF REQUESTED, THE PADDED FILE):

        real_head(38) = v0  
        real_head(39) = d0
        
        if (filter_type(1:nc_filter_type) == 'NF') then  ! no filter
          ncomments = ncomments + 1
          comments(ncomments) = 
     :      '| No filter'
          int_head(16) = ncomments
        else if (filter_type(1:nc_filter_type) == 'NOTCH') then  ! notch filter
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          ncomments = ncomments + 1
          comments(ncomments) = 
     :      '| File resulting from notch filtering with'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.4,1x,es10.3,1x,i1)') 
     :      '|  fc, d, icaus (0=acausal) = ', 
     :          flc, d4notch, icaus_bn
          real_head(33) = flc
          real_head(35) = d4notch
          int_head(16) = ncomments
        else if (filter_type(1:nc_filter_type) == 'LC') then
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          ncomments = ncomments + 1
          comments(ncomments) = 
     :      '| File resulting from low-cut Butterworth filter with:'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.4,1x,i2,1x,i1)') 
     :      '|  flc, nslope, icaus (0=acausal) = ', 
     :          flc, nslope_lc, icaus_lc
            real_head(33) = flc
            real_head(34) = -nroll_lc
          int_head(16) = ncomments
        else if (filter_type(1:nc_filter_type) == 'HC') then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '| File resulting from low-cut Butterworth filter with:'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,1x,i2,1x,i1)') 
     :      '|  fhc, nslope, icaus (0=acausal) = ', 
     :          fhc, nslope_hc, icaus_hc
          real_head(35) = fhc
          real_head(36) = -nroll_hc
          int_head(16) = ncomments
        else if (filter_type(1:nc_filter_type) == 'BAND') then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|  band-pass Butterworth filter with '
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.4,1x,f7.3)') 
     :      '|  with flc, fhc = ', flc, fhc
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,i2,1x,i1)') 
     :      '|  and slope, icaus = (0=acausal)', 
     :         nslope_bp, icaus_bn
          real_head(33) = flc
          real_head(34) = -nroll_bp
          real_head(35) = fhc
          real_head(36) = -nroll_bp
          int_head(16) = ncomments
        else if (filter_type(1:nc_filter_type) == 'LCHC') then  ! locut followed by hicut
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|  low-cut Butterworth filter with '
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.4,1x,i2,1x,i1)') 
     :      '|  flc, nslope, icaus (0=acausal) = ', 
     :          flc, nslope_lc, icaus_lc
          real_head(33) = flc
          real_head(34) = -nroll_lc
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '| followed by high-cut Butterworth filter with '
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,1x,i2,1x,i1)') 
     :      '|  fhc, nslope, icaus (0=acausal) = ', 
     :          fhc, nslope_hc, icaus_hc
          real_head(35) = fhc
          real_head(36) = -nroll_hc
          int_head(16) = ncomments
        else
          write(*,'(a)') 
     :           ' In write headers part of blpadflt, unknown '//
     :           'filter type ('//filter_type(1:nc_filter_type)//
     :           '); quitting!!! '
        end if
          
        if (filter_type(1:nc_filter_type) /= 'NF') then   
          if (compute_v_d == 'Y') then
            ncomments = ncomments + 1
            comments(ncomments) = 
     :      '| filtering acceleration before integration'
            int_head(16) = ncomments
          end if
        end if

        ncomments = int_head(16)

        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        write(comments(ncomments),'(a,2(1x,i7))') 
     :    '|  npts_in, npts_out (for filtered time series) = ',
     :        npts, npts_out

        if (compute_v_d == 'Y') then
          ncomments = ncomments + 1
          comments(ncomments) = 
     :       '| v0, d0 not used in integrating to vel, dis'
        end if

        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments

        npts_out_no_pads = npts_out - nlz - ntz
!        npts_out_no_pads = npts
!        npts_out_no_pads = izct - izcl + 1

        allocate (y_out(npts_out_no_pads))

        int_head(17) = npts_out_no_pads
       
        f_stem = ' '
        f_stem = f_in(1:nc_f_in)//string_to_append(1:nc_s2appnd)
        call trim_c(f_stem, nc_f_stem)


        do i = 1, npts_out_no_pads
!          y_out(i) = a_out(i+nlz-izcl+1)
          y_out(i) = a_out(i+nlz)
        end do
        
        path = ' '
        call get_path_from_file_name(f_stem,nc_f_stem,path,nc_path)

        f_a = ' '
        f_a = f_stem(nc_path+1:nc_f_stem)//'.a'
        if (compute_v_d == 'Y') then
          itype = 2  ! acceleration
        else
          itype = 0  ! unknown
        end if

        call trim_c(f_a,nc_f_a)
        write(nu_sum,'(2a)') 
     :        ' Writing output file (no pads): ', f_a(1:nc_f_a)
        write(nu_sum,'(3x,a, i6,a, 2(1x,es15.7))') 
     :     'a(1), a(',npts_out_no_pads,') = ',       
     :       y_out(1), y_out(npts_out_no_pads) 
     
        precision = ' '        
        if (int_head(47) == 8) then
            precision = 'HIGH'
        else
          if (high_precision4output(1:nc_high_precision4output) == 'Y') 
     :                                   then
            precision = 'HIGH'
          else
            precision = 'STANDARD'
          end if
        end if
        call trim_c(precision,nc_precision)

        call SMCWrite(f_a(1:nc_f_a), precision,
     :                y_out, char_head, itype, 
     :                    int_head, real_head, comments)

        
        if (compute_v_d == 'Y') then
          itype = 3  ! velocity

          do i = 1, npts_out_no_pads
!            y_out(i) = v_out(i+nlz-izcl+1)
            y_out(i) = v_out(i+nlz)
          end do

          f_v = ' '
          f_v = f_stem(nc_path+1:nc_f_stem)//'.v'

          call trim_c(f_v,nc_f_v)
          write(nu_sum,'(2a)') ' Writing output file: ', f_v(1:nc_f_v)
          write(nu_sum,'(3x,a, i6,a, 2(1x,es15.7))') 
     :     'v(1), v(',npts_out_no_pads,') = ',       
     :       y_out(1), y_out(npts_out_no_pads) 

          call SMCWrite(f_v(1:nc_f_v), precision,
     :                    y_out, char_head, itype, 
     :                    int_head, real_head, comments)

          itype = 4  ! displacement

          do i = 1, npts_out_no_pads
!            y_out(i) = d_out(i+nlz-izcl+1)
            y_out(i) = d_out(i+nlz)
          end do

          f_d = ' '
          f_d = f_stem(nc_path+1:nc_f_stem)//'.d'

          call trim_c(f_d,nc_f_d)
          write(nu_sum,'(2a)') ' Writing output file: ', f_d(1:nc_f_d)
          write(nu_sum,'(3x,a, i6,a, 2(1x,es15.7))') 
     :     'd(1), d(',npts_out_no_pads,') = ',       
     :       y_out(1), y_out(npts_out_no_pads) 

          call SMCWrite(f_d(1:nc_f_d), precision,
     :                    y_out, char_head, itype, 
     :                    int_head, real_head, comments)

        end if

        if (save_padded_time_series .and. 
     :      (nlz > 0 .or. ntz > 0)) then

! Only save if pads were actually added.

          int_head(17) = npts_out
          
          int_head(31) = nlz + 1           ! index for first pad-stripped value
          int_head(32) = npts_out - ntz    ! index for last pad-stripped value
          

          if (compute_v_d == 'Y') then
            itype = 2  ! acceleration
          else
            itype = 0  ! unknown
          end if
          

          f_pad = ' '
          f_pad = f_a(1:nc_f_a)//'.pad'

          call trim_c(f_pad,nc_f_pad)
          write(nu_sum,'(2a)') 
     :      ' Writing output file: ', f_pad(1:nc_f_pad)

          call SMCWrite(f_pad(1:nc_f_pad), precision,
     :                    a_out, char_head, itype, 
     :                    int_head, real_head, comments)

          if (compute_v_d == 'Y') then
            itype = 3  ! velocity

            f_pad = ' '
            f_pad = f_v(1:nc_f_v)//'.pad'

            call trim_c(f_pad,nc_f_pad)
            write(nu_sum,'(2a)') 
     :      ' Writing output file: ', f_pad(1:nc_f_pad)

            call SMCWrite(f_pad(1:nc_f_pad), precision,
     :                    v_out, char_head, itype, 
     :                    int_head, real_head, comments)

            itype = 4  ! displacement

            f_pad = ' '
            f_pad = f_d(1:nc_f_d)//'.pad'

            call trim_c(f_pad,nc_f_pad)
            write(nu_sum,'(2a)') 
     :      ' Writing output file: ', f_pad(1:nc_f_pad)

            call SMCWrite(f_pad(1:nc_f_pad), precision,
     :                    d_out, char_head, itype, 
     :                    int_head, real_head, comments)
          end if

        end if

        write(*,*)

! Step 8: COMPUTE RESPONSE SPECTRA
        if (nperiods /= 0) then

! Sinc interpolation:

          npts4work = interpolate_factor * npts_out

          allocate(work(npts4work))

          call interpolate_time_series_using_fork(
     :     a_out, sps, npts_out, 
     :     interpolate_factor,
     :     work, sps_work, npts_work
     :     )

          dt_work = 1.0/sps_work
        
          write(nu_sum,'(a)') 
     :     ' interpolate_factor, sps, sps_work, dt, dt_work, '//
     :      'npts_out, npts_work = '
          write(nu_sum, *) 
     :       interpolate_factor, sps, sps_work, dt, dt_work, 
     :       npts_out, npts_work

!DEBUG:
           write(nu_sum,*)
           write(nu_sum,'(a)') 
     :        ' npts_work, dt_work, use_straightline_interp, '//
     :         'work(1), work(npts_work)'
           write(nu_sum,'(1x,i6, 1x,es11.4, 1x,l1, 
     :                    1x,es11.4, 1x,es11.4)') 
     :          npts_work, dt_work, use_straightline_interp,
     :          work(1), work(npts_work)
!DEBUG:
          

! Find time of acceleration maximum:
          call mnmaxidx(work, 1, npts_work, 1,
     :                 amin,amax,indx_min,indx_max) 
          imax = indx_min
          if (abs(amax) > abs(amin)) then
            imax = indx_max
          end if
          tmax = float(imax - 1)*dt
          write(nu_sum,'(a)') 
     :     ' amin,amax,indx_min,indx_max,imax,tmax = '
          write(nu_sum,'(1p2(1x,e10.3),0p3(1x,i6),1p1x,e10.3)') 
     :       amin,amax,indx_min,indx_max,imax,tmax

! Set up periods:
        
          if (nperiods > 1) then
            dlogt = alog10(per_high/per_low)/float(nperiods-1)
          else
            dlogt = 0.0
          end if

          do i = 1, nperiods
            period(i) = per_low*10.0**(float(i-1) * dlogt)
          end do

! Open output file, set up and write column labels

          call get_lun(nu_rs)
          f_rs = f_stem(nc_path+1:nc_f_stem)//'.r.col'
          call trim_c(f_rs,nc_f_rs)
          open(unit=nu_rs,file=f_rs(1:nc_f_rs),status='unknown')

          write(nu_rs,'(1x,a, 1x,i2, 1x,a, 1x,l1)') 
     :       '! interpolate_factor = ', interpolate_factor,
     :       '; use_straightline_interp = ', use_straightline_interp

          call trim_c(f_a,nc_ts)
          i_stem = index(f_a,'.')
          header(1) = 'per:________'
          header(2) = 'frq:________'
          if(i_stem > 9) then
            i_stem = 9
          end if
          header(1)(5:i_stem+3) = f_a(1:i_stem-1) 
          header(2)(5:i_stem+3) = f_a(1:i_stem-1) 
          
          do i = 1, ndamp
            write(damp_c,'(i2.2)') nint(100.0*damp(i))
            header(5*(i-1)+3) = 'sd'//damp_c//'________'
            header(5*(i-1)+3)(5:i_stem+3) = f_a(1:i_stem-1) 
            header(5*(i-1)+4) = 'pv'//damp_c//'________'
            header(5*(i-1)+4)(5:i_stem+3) = f_a(1:i_stem-1) 
            header(5*(i-1)+5) = 'pa'//damp_c//'________'
            header(5*(i-1)+5)(5:i_stem+3) = f_a(1:i_stem-1) 
            header(5*(i-1)+6) = 'rv'//damp_c//'________'
            header(5*(i-1)+6)(5:i_stem+3) = f_a(1:i_stem-1) 
            header(5*(i-1)+7) = 'aa'//damp_c//'________'
            header(5*(i-1)+7)(5:i_stem+3) = f_a(1:i_stem-1) 
          end do

          write(nu_rs,'(27(1x,a12))') (header(i),i=1,5*ndamp+2)

! Loop over periods and damping, computing and writing response spectra:

          if (save_padded_time_series .and. 
     :       (nlz > 0 .or. ntz > 0)) then
             write(*,'(a)') 
     :     ' Patience! Computing rd for ts '//f_pad(1:nc_f_pad)
          else
             write(*,'(a)') 
     :     ' Patience! Computing rd for ts '//f_a(1:nc_f_a)
          end if
          write(*,*)
          do iper = 1, nperiods
            write(*,'(a,1x,f9.3)') 
     :    '+ and period ', period(iper)


            d04rs = 0.0
            v04rs = 0.0
            do idamp = 1, ndamp
              omega = twopi/period(iper)
              omega2 = omega**2
              call rs_calc(work, npts_work, omega, 
     :                   damp(idamp), dt_work, 
     :                   v04rs, d04rs, use_straightline_interp, 
     :                   rd(idamp), rv(idamp), aa(idamp) )
            end do ! loop over damping
          
            
            if (period(iper) == 0.0) then
              freq4rs = 999.99
            else if (period(iper) < 0.0) then
              freq4rs = period(iper)
            else
              freq4rs = 1.0/period(iper)
            end if
            
            write(nu_rs,'(1p27(1x,e12.5))')
     :      period(iper), freq4rs, (rd(i), omega*rd(i), omega2*rd(i), 
     :                     rv(i), aa(i), i = 1, ndamp) 
     
          end do ! loop over nperiods
          
          deallocate(work)

          close(nu_rs)
          
        end if

        deallocate (y_out)  ! must keep this before the rest
                            ! because the program can move to 6245 
                            ! before y_out is allocated if t2b >= tend 
6245    continue
        deallocate(a_in)
        deallocate (x, y, sig, time_r)
        deallocate (a_bl)
        deallocate (a_out, v_out, d_out)

      END DO loop over input lines

      close(nu_ctl)
      close(nu_sum)

      stop
      end

! ------------------- BEGIN D_BASELINE -----------
      function d_baseline(t1,t2,tend,v0,af)
        vt2 = v0+af*t2
        d_baseline = 0.5*vt2*(t2-t1) + vt2*(tend-t2)
     :              +0.5*af*(tend-t2)**2
! Comment (10/06/99):  
! The code between the comment lines is wrong.
! It was inserted after comparing runs using option 2 and option 0, with 
! t1 = t2.  For a particular run the final displacement was much larger using
! option 2 than using option 0 with t1 = t2.  I then did a series of trials
! using option 0 and t2 incrementally larger than t1.  From these runs I
! decided that the code below was correct.  But it isn't, because it ignores
! the impulse in acceleration due to the step in velocity at t = t1.  In effect,
! the code below assumes that the velocity line goes to zero at t = t1 if 
! t1 = t2.  The apparent inconsistency (d_final using the wrong code less than
! that using the proper code with option 2, which chooses t2 to minimize 
! d_final) is explained by considering the velocity.  Using the code below
! results in a velocity that is not as close to zero as it is when using the 
! proper code.  Note that the effect of the code below for t1 = t2 can be
! obtained by using option 0 and t1 = t2.
!        if (t1 == t2) then
!          d_baseline = 0.5*af*(tend-t2)**2
!        else
!          d_baseline = 0.5*vt2*(t2-t1) + vt2*(tend-t2)
!     :              +0.5*af*(tend-t2)**2
!        end if
! Comment (10/06/99):  
        return
      end
! ------------------- END D_BASELINE -----------

! ------------------- BEGIN A1_TO_T1 -----------
      subroutine a1_to_t1(a_out, npts, dt, a1, t1)
      real a_out(*)
      do i = 1, npts
        if (abs(a_out(i)) >= a1) then
          t1 = float(i-1) * dt
          goto 1111
        end if
      end do
      write(*, '(a)') '  amps all less than a1; quitting!!!'
      t1 = -999.0
1111  continue
      return
      end
! ------------------- END A1_TO_T1 -----------

! ------------------- BEGIN A2_TO_T2 -----------
      subroutine a2_to_t2(a_out, npts, dt, a2, t2)
      real a_out(*)
      do i = npts, 1, -1
        if (abs(a_out(i)) >= a2) then
          t2 = float(i-1) * dt
          goto 1111
        end if
      end do
      write(*, '(a)') '  amps all less than a1; quitting!!!'
      t2 = -999.0
1111  continue
      return
      end
! ------------------- END A2_TO_T2 -----------



      include 'blpadflt_util_subs.for'
 

                                          