* ---------------------------------------------------------------- GmeanRot
      Program GmeanRot

* Reads an SMC filename from a control file, loops over a series of rotation 
* angles, for each computing the geometric mean of the two horizontal
* components.
 
* Structure of control file:
* Line 1 follows:
*!Control file for program GMeanRot
*!Summary file name:
* gmeanrot.sum
*!dazm, azm_low, azm_high
*  1.0 0.0 89.0
*!step to use in unwrapping dazm
* 60.0
*!Periods for response spectra (nperiods log spaced from per_low to per_high):
* 200 0.01 90
*!Damping value (damp):
* 0.05 
*!nper for azimuth variation:
* 4
*! periods for az. variation:
* 0.1 0.3 1.0 3.0
*! fractions to use for fractiles:
*  0.25 0.5 0.75
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*! Extension for output file (append to end of first file name)
* gmnrot90.asc
*! list of file names (assumes pairs of files)  
*   0516a.smc
*   0516c.smc
*   0526a.smc
*   0526c.smc
* StoP

* Sign conventions:  the azimuth of the radial component is 180+azimuth from
* station to reference point, and the transverse component is this radial 
* azimuth plus 90 degrees.  This should be equivalent to the convention
* in Aki and Richards, Fig. 4.10, p. 96.

* This is incomplete, because the program will eventually also deal with case
* for which i_spcfy_fname = 1.  A program that works 
* with this is smcrtold (the old version of smc_rot)

* Dates: 12/23/04 - Written by D. Boore, based on smc2rs and smc_rot
*        12/28/04 - compute avg, std of logs of spectra,  Also, write out 
*                   psa and sd.
*        01/02/05 - Output different measures of the spread
*        01/03/05 - Unwrap dazm
*        01/18/05 - fractile percentage in col headers was one unit too low
*        01/20/05 - Add more output
*        01/20/05 - Add test to skip only if npts differ by more than 
*                   a specified amount.
*        06/16/05 - Put in datetime calcs to see how long it takes to do
*                   the computation (to check with a later version
*                   in which the rs is only computed twice for each
*                   azimuth).
*        09/20/05 - Use indexx as a means of selecting the 50th percentile
*                   value, to check the output of select
*        09/21/05 - Changed azimuth specification (dazm, azm_low, azm_high),
*                   one reason to be sure to avoid computations for azm = 0.0
*                   90.0, which are the same
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
  
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(20)*80, path_in*40, path_out*40      
       
      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)
      integer indx(200)

      real real_head(50,2), work(200), azm_added(200)
      real alat_sta(2), along_sta(2)
      real a_in(:,:), a_out(:,:)
      allocatable :: a_in, a_out
      
      real period(800), psa(2), gmean(800,200),
     :     gmean_min(800),dazm_gmnmin(800),dazm_gmnmin_unwrapped(800),
     :     gmean_max(800),dazm_gmnmax(800),dazm_gmnmax_unwrapped(800),
     :     psa_min(800),dazm_psamin(800),dazm_psamin_unwrapped(800),
     :     psa_max(800),dazm_psamax(800),dazm_psamax_unwrapped(800), 
     :     gm50(800)
      real omega, omega2
      real per4az(10)
      integer ndx4az(10)
         

      real rnull(2), azm_in(2), azm_out(2), sps(2)
      character buf_c*80, buf4_c*4, f_ctl*40, f_sum*40,
     :          buf_out*79
      character  f_in(2)*60, f_out_psa*60, f_out_sd*60, f_ext*30,
     :           c_az*7, rc(2)*12, f_out_work*60, f_az_psa*60,
     :           f_az_sd*60, f_az_per*60
      character per4az_c*5, colhead1(10)*12, colhead2(10)*12, 
     :          frmt_c*80
      
      logical f_ctl_exist, f_smc_exist

      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = gmeanrot.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'gmeanrot.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program GMeanRot'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) dazm, azm_low, azm_high
      write(nu_sum,*) dazm, azm_low, azm_high
      nazm = (azm_high - azm_low)/dazm + 1.3
      write(nu_sum,'(a,1x,i3)') ' nazm = ', nazm
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) step4azm_unwrap
      write(nu_sum,*) step4azm_unwrap
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nperiods, per_low, per_high
      write(nu_sum,*) nperiods, per_low, per_high
* Set up periods:
        
      dlogt = alog10(per_high/per_low)/float(nperiods-1)
      do i = 1, nperiods
        period(i) = per_low*10.0**(float(i-1) * dlogt)
      end do 
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nper4az
      write(nu_sum,*) nper4az
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) (per4az(i),i=1,nper4az)
      write(nu_sum,*) (per4az(i),i=1,nper4az)
      
      do i = 1, nper4az
        do j = 1, nperiods-1
          if (per4az(i) .gt. period(j) .and.
     :        per4az(i) .lt. period(j+1)) then
            ndx4az(i) = j
            go to 7777
          end if
        end do
        
        write(*,'(a)') ' ERROR: per4az not found in period'//
     :                 ' i, per4az(i) = '
        write(*,*) i, per4az(i)
        close(nu_ctl)
        stop
7777    continue
      end do
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) f1, f2, f3
      write(nu_sum,*) f1, f2, f3
      
      n1 = int(f1 * float(nazm))
      n2 = int(f2 * float(nazm))
      n3 = int(f3 * float(nazm))
 
      write(*,'(a)') ' n1, n2, n3 ='
      write(*,*)       n1, n2, n3
      
      write(nu_sum,'(a)') ' n1, n2, n3 ='
      write(nu_sum,*)       n1, n2, n3
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) npts_skip
      write(nu_sum,*) npts_skip
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_ext = ' '
      read(nu_ctl,'(a)') f_ext
      call trim_c(f_ext,nc_f_ext)
      write(nu_sum,'(a)') f_ext(1:nc_f_ext)
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

1000  continue ! Loop over files

        write(nu_sum,*)

 
* First get max dimension and allocate arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_ctl)
            stop
          end if

          call date_and_time(values=idate_time_values)
          time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
          inquire(file=f_in(j)(1:nc_f_in(j)), exist=f_smc_exist)
          if (.not. f_smc_exist) then
            write(*,'(a)') ' ******* FILE '//f_in(j)(1:nc_f_in(j))//
     :                   ' DOES NOT EXIST, QUITTING ******* '
            close(unit=nu_sum)
            close(nu_ctl)
            stop            
          end if
        
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
        end do

        max_dimen = max0(npts(1), npts(2))
        allocate( a_in(max_dimen,2), a_out(max_dimen,2))
        
        write(nu_sum,'(a,1x,i5)') ' max_dimen = ', max_dimen

* Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j))
          azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list

          do i = 1, npts(j)
            a_out(i,j) = a_in(i,j)
          end do
        end do

* Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
            go to 1000
        end if
* Check for equality of npts:
        if ( iabs(npts(1) - npts(2)) .gt. npts_skip) then
            write(nu_sum,'(a,3(1x,i6))') 
     :      ' SKIPPING: iabs(npts1 - npts2) > '//
     :      'npts_skip; npts1, npts2, npts_skip = ', 
     :        npts(1), npts(2), npts_skip
            go to 1000
        end if
* Check for equality of rnull:
        if ( rnull(1) .ne. rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' SKIPPING: rnull1 .ne. rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
            go to 1000
        end if
* Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
            go to 1000
        end if

* Obtain coordinates of stations from files and check for equality:
        do j = 1, 2
          alat_sta(j) = real_head(11,j)          
          along_sta(j) = real_head(12,j)          
        end do
        if (alat_sta(1) .ne. alat_sta(2) .or. 
     :      along_sta(1) .ne. along_sta(2) ) then
          write(*,'(a)') '  Station coordinates not equal, '//
     :                   ' see SUM file; SKIPPING!!!'
          write(nu_sum,'(a)') ' alat_sta(1), alat_sta(2) = '
          write(nu_sum,*)       alat_sta(1), alat_sta(2)
          write(nu_sum,'(a)') ' along_sta(1), along_sta(2) = '
          write(nu_sum,*)       along_sta(1), along_sta(2)
            go to 1000
        end if
        
* Passed all equality tests.

        write(nu_sum,'(a)') ' Passed all equality tests'
        
* Initialize summary variables:

        do iper = 1, nperiods
          gmean_min(iper) = 1.0e08
          gmean_max(iper) = 0.0
          dazm_gmnmin(iper)   = -999.0
          dazm_gmnmax(iper)   = -999.0
          dazm_gmnmin_unwrapped(iper)   = -999.0
          dazm_gmnmax_unwrapped(iper)   = -999.0
          psa_min(iper) = 1.0e08
          psa_max(iper) = 0.0
          dazm_psamin(iper)   = -999.0
          dazm_psamax(iper)   = -999.0
          dazm_psamin_unwrapped(iper)   = -999.0
          dazm_psamax_unwrapped(iper)   = -999.0
        end do
        
      
* Step: Remove a mean from the whole record

*  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
*          avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
 
* Step: Loop over azimuths:

        do iz = 1, nazm
        
          azm_added(iz) = float(iz-1)*dazm + azm_low
          az_to_rot_into = azm_in(1) + azm_added(iz)
        
          do j = 1, 2
            azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list
            do i = 1, npts(j)
              a_out(i,j) = a_in(i,j)
            end do
          end do
  
          call Rotate(a_out(1,1),a_out(1,2),rnull(2), 
     :      azm_out(1),azm_out(2),az_to_rot_into,npts(2))
     
* Compute response spectra:

          do iper = 1, nperiods
            buf_out = ' '
            buf_out = '+ Patience! Computing rd for azm = '
            write(buf_out(36:40),'(f5.1)') azm_in(1)
            buf_out(41:43) = ' + '
            write(buf_out(44:48),'(f5.1)') azm_added(iz)
            buf_out = buf_out(1:48)//' and period '
            write(buf_out(61:68),'(f8.3)') period(iper) 
            write(*,'(a)') buf_out
            omega = twopi/period(iper)
            omega2 = omega**2
            do j = 1, 2
              dt = 1.0/sps(j)
              d0 = 0.0
              v0 = 0.0
              call rdrvaa(a_out(1,j), npts(j), omega, 
     :                  damp, dt, rd, rv, aa, d0, v0)
              psa(j) = omega2 * rd 
            end do
            gmean(iper, iz) = sqrt(psa(1) * psa(2))
            if (gmean(iper, iz) .lt. gmean_min(iper)) then
              gmean_min(iper) = gmean(iper, iz)
              dazm_gmnmin(iper) = az_to_rot_into - azm_in(1)
            end if
            if (gmean(iper, iz) .gt. gmean_max(iper)) then
              gmean_max(iper) = gmean(iper, iz)
              dazm_gmnmax(iper) = az_to_rot_into - azm_in(1)
            end if          

            psamin12 = amin1(psa(1),psa(2))
            if (psamin12 .lt. psa_min(iper)) then
              psa_min(iper) = psamin12
              dazm_psamin(iper) = az_to_rot_into - azm_in(1)
            end if
            psamax12 = amax1(psa(1),psa(2))
            if (psamax12 .gt. psa_max(iper)) then
              psa_max(iper) = psamax12
              dazm_psamax(iper) = az_to_rot_into - azm_in(1)
            end if          
          end do ! loop over nperiods
        
        end do  ! end loop over azimuths
      
* Compute and output various things:      
*OUTPUT  period, gmean_avg, gmean_std, gmean_min, az_gmean_min, gmean_max, az_gmean_max

        f_out_psa = ' '
        f_out_psa = f_in(1)(1:nc_f_in(1))//'.psa.'//f_ext(1:nc_f_ext)
        call trim_c(f_out_psa,nc_f_out_psa)     
        call get_lun(nu_out_psa)
        open(unit=nu_out_psa,file=f_out_psa(1:nc_f_out_psa),
     :                                status='unknown')      

        write(nu_out_psa,'(
     :                 3x,a, 9x,a,
     :                 4x,a, i2.2,
     :                 4x,a, i2.2,
     :                 4x,a, i2.2,
     :                 3x,a, 5x,a, 5x,a, 
     :                 3x,a, 5x,a, 
     :                 6x,a, 5x,a, 
     :                 6x,a, 5x,a,
     :                 1x,a,
     :                 4x,a,
     :                 4x,a,
     :                 4x,a, 
     :                 2x,a,
     :                 3x,a,
     :                 1x,a,
     :                 8x,a,
     :                 9x,a,
     :                 1x,a
     :                        )') 
     :                      'period', 'azm1', 
     :                      'psagmn_', int(100.0*f1+0.5), 
     :                      'psagmn_', int(100.0*f2+0.5),
     :                      'psagmn_', int(100.0*f3+0.5),
     :                      'psagmn_min', 'dazwrapd', 'dazgmnmn', 
     :                      'psagmn_max', 'dazgmnmx',
     :                      'psa_min', 'dazpsamn',
     :                      'psa_max', 'dazpsamx',
     :                      '(gmn3-1)/gm2',
     :                      'gmn3/gmn1', 
     :                      'gmn2/gmn1', 
     :                      'gmn3/gmn2', 
     :                      'gmnmx/gmnmn',
     :                      'gmn2/gmnmn',
     :                      'gmnmx/psamax',
     :                      'psv50',
     :                      'sd50',
     :                      'gmpsa_record'
      
        f_out_sd = ' '
        f_out_sd = f_in(1)(1:nc_f_in(1))//'.sd.'//f_ext(1:nc_f_ext)
        call trim_c(f_out_sd,nc_f_out_sd)     
        call get_lun(nu_out_sd)
        open(unit=nu_out_sd,file=f_out_sd(1:nc_f_out_sd),
     :                                status='unknown')      
        write(nu_out_sd,'(
     :                 3x,a, 9x,a,
     :                 5x,a, i2.2,
     :                 5x,a, i2.2,
     :                 5x,a, i2.2,
     :                 4x,a, 5x,a, 5x,a, 
     :                 4x,a, 5x,a, 
     :                 7x,a, 5x,a, 
     :                 7x,a, 5x,a,
     :                 1x,a,
     :                 4x,a,
     :                 4x,a,
     :                 4x,a,
     :                 2x,a,
     :                 3x,a,
     :                 2x,a,
     :                 8x,a,
     :                 8x,a,
     :                 1x,a
     :                        )') 
     :                      'period', 'azm1', 
     :                      'sdgmn_', int(100.0*f1+0.5), 
     :                      'sdgmn_', int(100.0*f2+0.5),
     :                      'sdgmn_', int(100.0*f3+0.5),
     :                      'sdgmn_min', 'dazwrapd','dazgmnmn',
     :                      'sdgmn_max', 'dazgmnmx',
     :                      'sd_min', 'dazgmnmn', 
     :                      'sd_max', 'dazpsamx',
     :                      '(gmn3-1)/gm2',
     :                      'gmn3/gmn1',
     :                      'gmn2/gmn1',
     :                      'gmn3/gmn2',
     :                      'gmnmx/gmnmn',
     :                      'gmn2/gmnmn',
     :                      'gmnmx/sdmax',
     :                      'psv50',
     :                      'psa50',
     :                      'gm_sd_record'
      
        f_az_per = ' '
        f_az_per = f_in(1)(1:nc_f_in(1))//'.az_per.'//
     :             f_ext(1:nc_f_ext)
        call trim_c(f_az_per,nc_f_az_per)     
        call get_lun(nu_az_per)
        open(unit=nu_az_per,file=f_az_per(1:nc_f_az_per),
     :                                status='unknown') 
     
        write(nu_az_per,'(7x,a, 4x,a, 1x,a)')
     :        'per', 'gmrot50', 'azgmrot50'

        do iper = 1, nperiods
          dazm_gmnmin_unwrapped(iper) = dazm_gmnmin(iper)         
          dazm_gmnmax_unwrapped(iper) = dazm_gmnmax(iper)         
          dazm_psamin_unwrapped(iper) = dazm_psamin(iper)         
          dazm_psamax_unwrapped(iper) = dazm_psamax(iper)
        end do
      
        do iper = 1, nperiods
      
          do iz = 1, nazm
            work(iz) =  gmean(iper, iz)
          end do
      
          call indexx(nazm, work, indx)
          gmrot50 = work(indx(n2))
          azgmrot50 = azm_added(indx(n2))
          
          write(nu_az_per,'(1x,f9.5, 1x,1p,e10.3, 0p,4x,f6.1)')
     :                     period(iper), gmrot50, azgmrot50
          
          
          
          psagmn1 = select(n1, nazm, work)
          psagmn2 = select(n2, nazm, work)
          psagmn3 = select(n3, nazm, work)
          
          gm50(iper) = psagmn2  ! assume n2 corresponds to 50 % fractile
        
          if (iper .gt. 1) then
            diff_az_gmnmin = 
     :        dazm_gmnmin_unwrapped(iper)-dazm_gmnmin_unwrapped(iper-1)
            if(abs(diff_az_gmnmin) .gt. step4azm_unwrap) then
              dazm_gmnmin_unwrapped(iper) = 
     :          dazm_gmnmin_unwrapped(iper) - 
     :                            sign(90.0, diff_az_gmnmin)
            end if
            diff_az_gmnmax = 
     :        dazm_gmnmax_unwrapped(iper)-dazm_gmnmax_unwrapped(iper-1)
            if(abs(diff_az_gmnmax) .gt. step4azm_unwrap) then
              dazm_gmnmax_unwrapped(iper) = 
     :          dazm_gmnmax_unwrapped(iper) - 
     :                            sign(90.0, diff_az_gmnmax)
            end if
            diff_az_psamin = 
     :        dazm_psamin_unwrapped(iper)-dazm_psamin_unwrapped(iper-1)
            if(abs(diff_az_psamin) .gt. step4azm_unwrap) then
              dazm_psamin_unwrapped(iper) = 
     :          dazm_psamin_unwrapped(iper) - 
     :                            sign(90.0, diff_az_psamin)
            end if
            diff_az_psamax = dazm_psamax(iper)-dazm_psamax(iper-1)
            if(abs(diff_az_psamax) .gt. step4azm_unwrap) then
              dazm_psamax(iper) = dazm_psamax(iper) - 
     :                            sign(90.0, diff_az_psamax)
            end if
          end if
             
            
          write(nu_out_psa,'(1x,f8.4, 1p, 24(1x,e12.5))')
     :                 period(iper), azm_in(1), 
     :                 psagmn1, psagmn2, psagmn3, 
     :                 gmean_min(iper), dazm_gmnmin(iper),
     :                                  dazm_gmnmin_unwrapped(iper),
     :                 gmean_max(iper), dazm_gmnmax_unwrapped(iper),
     :                 psa_min(iper), dazm_psamin_unwrapped(iper),
     :                 psa_max(iper), dazm_psamax_unwrapped(iper),
     :                 (psagmn3-psagmn1)/psagmn2,
     :                 psagmn3/psagmn1,
     :                 psagmn2/psagmn1,
     :                 psagmn3/psagmn2,
     :                 gmean_max(iper)/gmean_min(iper),
     :                 psagmn2/gmean_min(iper),
     :                 gmean_max(iper)/psa_max(iper),
     :                 psagmn2*(period(iper)/twopi),
     :                 psagmn2*(period(iper)/twopi)**2,
     :                 gmean(iper,1)
     
          omega      = twopi/period(iper)
          omega2     = omega**2
          sdgmn1      = psagmn1/omega2
          sdgmn2      = psagmn2/omega2
          sdgmn3      = psagmn3/omega2
          sd_gmn_min = gmean_min(iper)/omega2
          sd_gmn_max = gmean_max(iper)/omega2
          sd_ind_min = psa_min(iper)/omega2
          sd_ind_max = psa_max(iper)/omega2
      
          write(nu_out_sd,'(1x,f8.4, 1p, 24(1x,e12.5))')
     :                 period(iper), azm_in(1), 
     :                 sdgmn1, sdgmn2, sdgmn3,
     :                 sd_gmn_min, dazm_gmnmin(iper),
     :                             dazm_gmnmin_unwrapped(iper),
     :                 sd_gmn_max, dazm_gmnmax_unwrapped(iper),
     :                 sd_ind_min, dazm_psamin_unwrapped(iper),
     :                 sd_ind_max, dazm_psamax_unwrapped(iper),
     :                 (sdgmn3-sdgmn1)/sdgmn2,
     :                 sdgmn3/sdgmn1,
     :                 sdgmn2/sdgmn1,
     :                 sdgmn3/sdgmn2,
     :                 sd_gmn_max/sd_gmn_min,
     :                 sdgmn2/sd_gmn_min,
     :                 sd_gmn_max/sd_ind_max,
     :                 sdgmn2*omega,
     :                 sdgmn2*omega2,
     :                 gmean(iper,1)/omega2
     
     
        end do
      
        close(nu_az_per)
        close(nu_out_psa)
        close(nu_out_sd)

        call get_lun(nu_az_psa)
        f_az_psa = ' '
        f_az_psa = f_in(1)(1:nc_f_in(1))//'.az_psa.'//f_ext(1:nc_f_ext)
        call trim_c(f_az_psa,nc_f_az_psa)     
        open(unit=nu_az_psa,file=f_az_psa(1:nc_f_az_psa),
     :                                status='unknown')      
      
* Set up and write column headers of output file:

        do i = 1, nper4az
          per4az_c = ' '
          write(per4az_c,'(f5.2)') period(ndx4az(i))
          call trim_c(per4az_c,nc_per4az_c)
          colhead1(i) = ' '
          write(colhead1(i)(1:12),'(a12)') 'gm50per00000'
          colhead1(i)(12-nc_per4az_c+1:12) = per4az_c(1:nc_per4az_c)
          colhead2(i) = ' '
          write(colhead2(i)(1:12),'(a12)') 'gmazper00000'
          colhead2(i)(12-nc_per4az_c+1:12) = per4az_c(1:nc_per4az_c)
        end do
      
        frmt_c = ' '
        frmt_c = '(1x,a, 2x,a, xx(1x,a12, 1x,a12))' 
        write(frmt_c(14:15), '(i2)') nper4az
        call trim_c(frmt_c,nc_frmt_c)

        write(nu_az_psa,frmt_c(1:nc_frmt_c)) 
     :   'azm_in(1)', 'dazimuth', 
     :   (colhead1(i), colhead2(i), i = 1, nper4az)     
      
        do iz = 1, nazm
           write(nu_az_psa, '(2(2x,f8.4),1p,24(1x,e12.5))')
     :       azm_in(1), azm_added(iz), 
     :       (gm50(ndx4az(j)), gmean(ndx4az(j),iz), j=1,nper4az)
        end do
     
        close(nu_az_psa)
        deallocate(a_in)
        deallocate(a_out)
        
          call date_and_time(values=idate_time_values)
          time_sec_stop = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                    float(idate_time_values(8))/1000.0
     
       write(nu_sum,'(a,1x,f7.3)') ' calc time = ',
     :                     time_sec_stop - time_sec_start
     
      go to 1000    ! loop back for another pair of components

      end
* ---------------------------------------------------------------- SMC_Rot

      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\smcwrite.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
      include '\forprogs\indexx.for'
  

*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
*Write a file with az1, daz, (period, gmean, gmean-%, gmean_50%, gmean+%), repeated for a set of periods specficied in input  

       
