* ------------------------------------------------------------------- SMC2ENV
      Program SMC2ENV

* Uses the FFT to compute the envelope of a time series.  
* Stores the result in a column file.

* The output file name has the same stem name as the input file, with "e"
* as the second character of the extension, but without the path 
* (keeping the third letter, which often indicates component).  The exceptions
* to not altering the stem name are if the extension is "smc", then
* "smc" is replaced by "env".  In addition, if the last 2 characters of
* the stem name of the input file is "_A" or "_U" and the extension is "smc",
* last two characters of the stem name are replaced with "ae" or "ue" (I had 
* to replace "_" with either "a" or "u"
* because otherwise would have an identical filename for the spectra of
* corrected or uncorrected data, whereas might want them separately).  
* In this case, the extension = "env" (not "smc", because the file is not
* an smc file).

* Input from a control file; here is an example:
*!Control file for program SMC2ENV
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following 6 lines.  Note that ndec is used only if df_intrp = 0.0.
*!Name of summary file:
* smc2env.sum
*pp:
*  0.0 200.0 0.0   !tskip, tlength, frctnfrtaper
*  +1.0            !signnpw2(<0, backup for npw2, no zpad)
*    .true.        !dc_remove?
*    .false.       !detrend?
*  1               ! nsmooth_phd (need an entry even if not used)
* \psv\taiwan99\t0781pea.smc
* \psv\taiwan99\t1293l3v.smc
* \psv\taiwan99\t0781ped.smc
*pp:
*  0.0 90.0 0.05   !tskip, tlength, frctnfrtaper
*  1.0             !signnpw2(<0, backup for npw2, no zpad)
*    .true.        !dc_remove?
*    .false.       !detrend?
*  1               ! nsmooth_phd (need an entry even if not used)
* file1.smc
* file2.smc
* file3.smc
*stop

* Dates: 06/19/02 - Written by D.M. Boore, patterned after SMC2PHS
*        07/14/02 - Allow for smoothing of instantaneous frequency
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
!        07/11/13 - Allow for longer file names
!                 - Add ".env" to smc filename to make the output file name
!                 - Remove path stuff


c
c Dimension and declaration statements:

      real ts(:), envelope(:), freq_inst(:)
      allocatable :: ts, envelope, freq_inst

      integer npts, nc_f_smc
      real tskip, tlength, ftaper
      real signnpw2
      real t_smc, t_fft
      integer ios

      character date_c*8, time_c*11
      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*200, buf_upper*4, 
     :        f_smc*200, 
     :        f_env*200, f_sum*40, path*60, c2*2, c2up*2, 
     :        ext_c*10

      character f_ctl*40, colhead(4)*16
      logical f_ctl_exist, f_smc_exist
      logical dc_remove, detrend

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2ENV.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2env.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call trim_c(f_ctl, nc_f_ctl)

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2FS2:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      loop over smc files: DO
        buf = ' '
        read(nu_ctl,'(a)',iostat=ios) buf
        if (ios /= 0) then
          close (nu_ctl)
          EXIT loop over smc files
        end if
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf == 0 .or. buf(1:1) == '!') 
     :       CYCLE loop over smc files
        if(buf_upper(1:4) == 'STOP') EXIT loop over smc files
      
        if(buf_upper(1:3) == 'PP:') then  ! a new set of processing parameters
          read(nu_ctl,*) tskip, tlength_in, ftaper
          write(nu_sum,'(a)') 
     :       ' tskip, tlength_in, ftaper ='
          write(nu_sum,*) 
     :       tskip, tlength_in, ftaper

          read(nu_ctl,*) signnpw2
          write(nu_sum,'(a)') 
     :      ' signnpw2 = '
          write(nu_sum, *) 
     :      signnpw2

          read(nu_ctl,*) dc_remove
          write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove

          read(nu_ctl,*) detrend
          write(nu_sum,'(a,1x,l1)')  ' detrend = ', detrend

          read(nu_ctl,*) nsmooth_phd
          write(nu_sum,'(a,1x,i4)') ' nsmooth_phd = ', nsmooth_phd

          CYCLE loop over smc files
        end if

        f_smc = ' '
        f_smc = buf(1:nc_buf)
        nc_f_smc = nc_buf     
        inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
        if (.not. f_smc_exist) then
            write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
            CYCLE loop over smc files
        end if

* Passed tests; proceed

* Get number of points on this loop:

        call smc_npts(f_smc(1:nc_f_smc), npts, sps)

        write(nu_sum,*)
        write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

        call get_npw2(npts,signnpw2,npw2)

        n4ts_alloc = max0(npts, npw2)
        nout       = min0(npts, npw2)

        allocate( ts(n4ts_alloc), envelope(npw2), freq_inst(npw2) )

* Now read the data from the files:

        write(*,'(1x,a)') 
     :     'Processing file:', f_smc(1:nc_f_smc)

        tlength = tlength_in
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :                 ts,  
     :                 char_head, int_head, real_head, comments)
        sps = real_head(2)
        npts_smc = int_head(17)
        dt = 1.0/sps
        t_smc = float(npts_smc-1)*dt
        nacc_start = tskip * sps
        nacc_stop  = nacc_start + tlength * sps

        write(nu_sum,'(3x,a,f8.6,a,i6,a,f8.2)') 
     :       'signal_in: dt=', dt, 
     :       ' npts_smc=', npts_smc,' t_smc=', t_smc
        write(nu_sum,'(3x,a,1x,f8.2)') 
     :       ' tlength after smcread = ', tlength

* Condition the time series:

        call Condition(ts, dt, npts, npw2, 
     :                 ftaper, dc_remove, detrend, nzpad)      

* Compute the envelope and instantaneous frequency:

        call Envelope_SR(ts, dt, npw2, nsmooth_phd, 
     :          envelope, freq_inst) 

        t_fft = float(npw2-1)*dt
        
        write(nu_sum,'(3x,a,f8.6,a,i6,a,f8.2)')
     :     'signal_fft: dt=', dt, ' npw2=', npw2,' t_fft=', t_fft

* Write output:

* Construct output filename:
!        call get_path(f_smc,nc_f_smc,path,nc_path)
        f_env = ' '
        f_env = f_smc(1:nc_f_smc)//'.env'

        call trim_c(f_env,nc_f_env)

        call trim_c(f_env,nc_f_env)

        write(*,'(1x,a)') 
     :     'Writing output to file:', f_env(1:nc_f_env)
        write(nu_sum,'(2a)') ' Output file = ', f_env(1:nc_f_env)

        call get_lun(nu_env)
        nu_env = 10
        open(unit=nu_env,file=f_env,status='unknown')

        write(nu_env,'(a)') '! Output of program SMC2ENV'
        write(nu_env,'(a)') '! working on file '//f_smc(1:nc_f_smc)
        write(nu_env,'(a)') 
     :                '! using control file '//f_ctl(1:nc_f_ctl)

        iflag = 0
        call datetime(date_c, time_c, iflag)
        write(nu_env,'(a)') '!   file written on '//date_c//
     :                      ' at time '//time_c

        write(nu_env, '(a,1x,f7.2,1x,f8.2,1x,f4.1)') 
     :      '!   tskip, tlength_in, ftaper = ',
     :      tskip, tlength_in, ftaper

        write(nu_env, '(a)') 
     :    '!   npw2, nout, dt, nsmooth_phd = '
*        write(nu_env, '(1x,i6,1x,i6, 1x,f8.5, 1x,i6)') 
         write(nu_env,*)
     :         npw2, nout, dt, nsmooth_phd

        write(nu_env, '(a, 2(1x,l1))') 
     :    '!   dc_remove, detrend = ',
     :      dc_remove, detrend

        do i = 1, 4
          colhead(i) = '________________'
          colhead(i)(5:16) = f_env(1:12)
        end do
        colhead(1)(1:4) = 'ts_:'      
        colhead(2)(1:4) = 'env:'      
        colhead(3)(1:4) = 'en-:'      
        colhead(4)(1:4) = 'f_i:'      

        write(nu_env,'(9x,a,1x,a,1x,a,1x,a,1x,a)') 
     :         'time', (colhead(i),i=1,4)

        do j = 1, nout
          write(nu_env,'(1p,1x,e12.5,4(5x,e12.5))')
     :          float(j-1)*dt, ts(j), envelope(j), -envelope(j),
     :          freq_inst(j)
        end do

        close(nu_env)

        deallocate( ts, envelope, freq_inst )
        
      END DO loop over smc files

 
* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2ENV


      include '\forprogs\fork.for'
      include '\forprogs\tprfrctn.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\smths.for'
      include '\forprogs\locate.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\zeropad.for'
      include '\forprogs\conditn.for'
      include '\forprogs\envelope.for'
      include '\forprogs\ampphphd.for'
      include '\forprogs\smooth_n.for'
      include '\forprogs\get_path.for'
      include '\forprogs\datetime.for'


