* ---------------------------------------------------------------- SMC2GM_AR
      Program SMC2GM_AR

* Reads a pair of SMC filenames from a control file and writes out 
* the as-recorded (AR) geometric mean of the two horizontal
* components as well as the response spectra of each individual component
* and the larger of the two components. The output is PSA and SD in cgs units.
* PGA and PGV are also written to the output file (corresponding to periods 
* of 0.0 and -1.0, respectively).
*
 
* Structure of control file:
* Line 1 follows:
*!Control file for program SMC2GM_AR
*!Summary file name:
* smc2gm_ar.sum
*!Use PEER NGA periods for gmrot50 computation (yfull/ysub/n):
*!      data per_nga_full/
*!     :  -1.0, 0.0,   
*!     :   0.010,0.020,0.022,0.025,0.029,0.030,0.032,0.035,0.036,0.040,
*!     :   0.042,0.044,0.045,0.046,0.048,0.050,0.055,0.060,0.065,0.067,
*!     :   0.070,0.075,0.080,0.085,0.090,0.095,0.100,0.110,0.120,0.130,
*!     :   0.133,0.140,0.150,0.160,0.170,0.180,0.190,0.200,0.220,0.240,
*!     :   0.250,0.260,0.280,0.290,0.300,0.320,0.340,0.350,0.360,0.380,
*!     :   0.400,0.420,0.440,0.450,0.460,0.480,0.500,0.550,0.600,0.650,
*!     :   0.667,0.700,0.750,0.800,0.850,0.900,0.950,1.000,1.100,1.200,
*!     :   1.300,1.400,1.500,1.600,1.700,1.800,1.900,2.000,2.200,2.400,
*!     :   2.500,2.600,2.800,3.000,3.200,3.400,3.500,3.600,3.800,4.000,
*!     :   4.200,4.400,4.600,4.800,5.000,5.500,6.000,6.500,7.000,7.500,
*!     :   8.000,8.500,9.000,9.500,10.000/
*!      nper_nga_full = 107 
*!      
*!        
*!      data per_nga_sub/
*!     :   -1.000,
*!     :    0.000,
*!     :    0.010,
*!     :    0.020,
*!     :    0.030,
*!     :    0.050,
*!     :    0.075,
*!     :    0.100,
*!     :    0.150,
*!     :    0.200,
*!     :    0.250,
*!     :    0.300,
*!     :    0.400,
*!     :    0.500,
*!     :    0.750,
*!     :    1.000,
*!     :    1.500,
*!     :    2.000,
*!     :    3.000,
*!     :    4.000,
*!     :    5.000,
*!     :    7.500,
*!     :   10.000/
*!      nper_nga_sub = 23 
*!
* ysub
*!if "n", then specify periods: nperiods log spaced from per_low to per_high:
*!need these as placeholders, even if use NGA periods
* 200 0.01 90
*!Damping value (damp):
* 0.05
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*! Extension for output file (append to end of first file name)
* .gm_ar.asc
*! list of file names (assumes pairs of files) 
*    0516a.smc
*    0516c.smc
*    0526a.smc
*    0526c.smc
*   StoP
 
 
* Dates: 10/18/07 - Written by D. Boore, based on smc2psagmrot.for.
*        10/23/07 - Write the two file names as headers in the output file
*        11/12/09 - Add psa_gm output in g
   
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(200)*80, path_in*40, path_out*40      
       
      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)

      real real_head(50,2), work(400), azm_added(400)
      real alat_sta(2), along_sta(2)
      real a_in(:,:), vel(:), dis(:)
      allocatable :: a_in,  vel, dis 
       
      real period(300), rd(2), psa(2), pga(2), pgv(2), 
     :    gmean(300,400), gmean_norm(300,400),
     :    gmrotd50(400), penalty(400), gm50(400)
      real psa_1_ar(400), psa_2_ar(400), psa_larger_ar(400), 
     :      psa_gm_ar(400)
      real psamaxrot(400), psaminrot(400)
      real angle4psamax(400), angle4psamin(400)
      real omega, omega2
      real per4az(10)
      integer ndx4az(10)
         

      real rnull(2), azm_in(2), azm_out(2), azm4file(200), sps(2)
      character buf_c*80, buf4_c*4, f_ctl*80, f_sum*80,
     :          buf_out*79
      character  f_in(2)*80, f_out*100, f_ext*80, f_penalty*100,
     :           c_az*7, rc(2)*12
      character per4az_c*5, colhead1(10)*12, colhead2(10)*12, frmt_c*80
      character use_nga_per4gm*10
      logical f_ctl_exist
      logical rmv_trnd
      
      real per_nga_full(107), per_nga_sub(23)   ! include pgv and pga as per = -1.0, 0.0
      data per_nga_full/
     :  -1.0, 0.0,   
     :   0.010,0.020,0.022,0.025,0.029,0.030,0.032,0.035,0.036,0.040,
     :   0.042,0.044,0.045,0.046,0.048,0.050,0.055,0.060,0.065,0.067,
     :   0.070,0.075,0.080,0.085,0.090,0.095,0.100,0.110,0.120,0.130,
     :   0.133,0.140,0.150,0.160,0.170,0.180,0.190,0.200,0.220,0.240,
     :   0.250,0.260,0.280,0.290,0.300,0.320,0.340,0.350,0.360,0.380,
     :   0.400,0.420,0.440,0.450,0.460,0.480,0.500,0.550,0.600,0.650,
     :   0.667,0.700,0.750,0.800,0.850,0.900,0.950,1.000,1.100,1.200,
     :   1.300,1.400,1.500,1.600,1.700,1.800,1.900,2.000,2.200,2.400,
     :   2.500,2.600,2.800,3.000,3.200,3.400,3.500,3.600,3.800,4.000,
     :   4.200,4.400,4.600,4.800,5.000,5.500,6.000,6.500,7.000,7.500,
     :   8.000,8.500,9.000,9.500,10.000/
      nper_nga_full = 107 
      
        
      data per_nga_sub/
     :   -1.000,
     :    0.000,
     :    0.010,
     :    0.020,
     :    0.030,
     :    0.050,
     :    0.075,
     :    0.100,
     :    0.150,
     :    0.200,
     :    0.250,
     :    0.300,
     :    0.400,
     :    0.500,
     :    0.750,
     :    1.000,
     :    1.500,
     :    2.000,
     :    3.000,
     :    4.000,
     :    5.000,
     :    7.500,
     :   10.000/
      nper_nga_sub = 23 


      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0
      nper_dimen = 300
      nazm_dimen = 400

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc2gm_ar.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc2gm_ar.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program GMeanRot'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      use_nga_per4gm = ' '
      read(nu_ctl,'(a)') use_nga_per4gm
      call upstr(use_nga_per4gm)
      call trim_c(use_nga_per4gm,nc_use_nga_per4gm)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nper_specify, per_low, per_high
      write(nu_sum,*) nper_specify, per_low, per_high
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
       
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_ext = ' '
      read(nu_ctl,'(a)') f_ext
      call trim_c(f_ext,nc_f_ext)
      write(nu_sum,'(a)') f_ext(1:nc_f_ext)
         
* Set up periods:
        


      if(use_nga_per4gm(1:nc_use_nga_per4gm) .eq. 'YFULL') then
        nper_max = nper_nga_full - 2
        do i = 1, nper_max
          period(i) = per_nga_full(i+2)
        end do
      else if (use_nga_per4gm(1:nc_use_nga_per4gm) .eq. 'YSUB') then
        nper_max = nper_nga_sub - 2
        do i = 1, nper_max
          period(i) = per_nga_sub(i+2)
        end do
      else
        nper_max = nper_specify
        if (nper_max .gt. nper_dimen) then
          nper_max = nper_dimen
          nper_specify = nper_dimen
          write(nu_sum,'(a,1x,i3)') 
     :          ' Resetting nper_max to ', nper_max
          write(nu_sum,'(a,1x,i3)') 
     :          ' Resetting nper_specify to ', nper_specify
        end if
        dlogt = alog10(per_high/per_low)/float(nper_specify-1)
        do i = 1, nper_max
          period(i) = per_low*10.0**(float(i-1) * dlogt)
        end do   
      end if
      


      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

1000  continue ! Loop over files

        write(nu_sum,*)

 
* First get max dimension and allocate arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_ctl)
            stop
          end if

          call date_and_time(values=idate_time_values)
          time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
        
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
        end do
        
* Set nperiods:

        nperiods = nper_max
876     continue          
        max_dimen = max0( npts(1), npts(2) )
        allocate( a_in(max_dimen,2) )
        allocate( vel(max_dimen), dis(max_dimen) )
 
* Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j))
          
          if (j .eq. 1) then
            azm_in(j) = 0.0
          else
            azm_in(j) = 90.0
          end if
          
          azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list

        end do

* Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
            go to 1000
        end if
* Check for equality of rnull:
        if ( rnull(1) .ne. rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' SKIPPING: rnull1 .ne. rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
            go to 1000
        end if
* Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
            go to 1000
        end if

* Obtain coordinates of stations from files and check for equality:
        do j = 1, 2
          alat_sta(j) = real_head(11,j)          
          along_sta(j) = real_head(12,j)          
        end do
        if (alat_sta(1) .ne. alat_sta(2) .or. 
     :      along_sta(1) .ne. along_sta(2) ) then
          write(*,'(a)') '  Station coordinates not equal, '//
     :                   ' see SUM file; SKIPPING!!!'
          write(nu_sum,'(a)') ' alat_sta(1), alat_sta(2) = '
          write(nu_sum,*)       alat_sta(1), alat_sta(2)
          write(nu_sum,'(a)') ' along_sta(1), along_sta(2) = '
          write(nu_sum,*)       along_sta(1), along_sta(2)
            go to 1000
        end if
        
* Passed all equality tests.

      
* Step: Remove a mean from the whole record

*  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
*          avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
 
* Step: Loop over periods:

        do iper = 1, nperiods
        
* Compute as-recorded oscillator time series for this period:
          omega = twopi/period(iper)
          omega2 = omega**2
          do j = 1, 2
            dt = 1.0/sps(j)
            d0 = 0.0
            v0 = 0.0
            call rdrvaa(a_in(1,j), npts(j),
     :            omega,damp,dt,rd(j),rv,aa, d0, v0)  
           end do
          
          psa_1_ar(iper) = omega2*rd(1)
          psa_2_ar(iper) = omega2*rd(2)
          psa_gm_ar(iper) = sqrt(psa_1_ar(iper)*psa_2_ar(iper))
          if (psa_1_ar(iper) .gt. psa_2_ar(iper)) then
             psa_larger_ar(iper) = psa_1_ar(iper)
          else
             psa_larger_ar(iper) = psa_2_ar(iper)
          end if
 
 
          
        end do ! loop over nperiods
        
      
 
        
        do j=1, 2
          dt = 1.0/sps(j)
          v0 = 0.0
          d0 = 0.0
          rmv_trnd = .false.
          call acc2vd(a_in(1,j), npts(j), dt, rmv_trnd, 
     :         v0, d0, vel, dis)
          call abs_mnmaxidx(vel, 1, npts(j), 1,
     :             vmin, vmax, indxpgv_min, indxpgv_max, pgv(j)) 
 
          call abs_mnmaxidx(a_in(1,j), 1, npts(j), 1,
     :             amin, amax, indxpga_min, indxpga_max, pga(j)) 
        end do
        
        pga_gm = sqrt(pga(1)*pga(2))
        pgv_gm = sqrt(pgv(1)*pgv(2))
        if (pga(1) .gt. pga(2)) then
          pga_larger = pga(1)
        else
          pga_larger = pga(2)
        end if        
        if (pgv(1) .gt. pgv(2)) then
          pgv_larger = pgv(1)
        else
          pgv_larger = pgv(2)
        end if        

* Now write out the values:

        f_out = ' '
        f_out = f_in(1)(1:nc_f_in(1))//f_ext(1:nc_f_ext)
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(nu_out,file=f_out(1:nc_f_out),status='unknown')
        write(nu_sum,'(a)') ' Output file = '//f_out(1:nc_f_out)
        
        write(nu_out,'(a)') ' input file 1: '//f_in(1)(1:nc_f_in(1))
        write(nu_out,'(a)') ' input file 2: '//f_in(2)(1:nc_f_in(2))
        write(nu_out,'(6x,a, 
     :                 3x,a, 3x,a,
     :                 1x,a, 2x,a,
     :                 2x,a,
     :                 3x,a, 3x,a,
     :                 1x,a, 2x,a
     :                  )') 
     :    'per', 
     :    'psa_1_ar', 'psa_2_ar', 
     :    'psa_larger_ar', 'psa_gm_ar',
     :    'pagmar(g)',
     :    ' sd_1_ar', ' sd_2_ar', 
     :    ' sd_larger_ar', ' sd_gm_ar'
      
        write(nu_out,'(1x,f8.4, 
     :               1p,
     :                   1x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3
     :                                        )') 
     :            per_nga_full(1), 
     :                   pgv(1), pgv(2), 
     :                   pgv_larger, pgv_gm
        write(nu_out,'(1x,f8.4, 
     :               1p,
     :                   1x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3
     :                                        )') 
     :            per_nga_full(2), 
     :                   pga(1), pga(2), 
     :                   pga_larger, pga_gm
     
        do iper = 1, nperiods
          omega = twopi/period(iper)
          omega2 = omega**2
          write(nu_out,'(1x,f8.4, 
     :               1p,
     :                   1x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3,
     :                   1x,e10.3,
     :                   1x,e10.3, 1x,e10.3, 
     :                   4x,e10.3, 1x,e10.3
     :                                                      )') 
     :      period(iper), 
     :      psa_1_ar(iper), psa_2_ar(iper), 
     :      psa_larger_ar(iper), psa_gm_ar(iper),
     :      psa_gm_ar(iper)/981.0,
     :      psa_1_ar(iper)/omega2, psa_2_ar(iper)/omega2, 
     :      psa_larger_ar(iper)/omega2, psa_gm_ar(iper)/omega2
        end do

        close (nu_out)
        
        deallocate( a_in )
        deallocate( vel, dis )
         
       
        call date_and_time(values=idate_time_values)
        time_sec_stop = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                    float(idate_time_values(8))/1000.0
     
       write(nu_sum,'(a,1x,f7.3)') ' calc time = ',
     :                     time_sec_stop - time_sec_start
     
      go to 1000    ! loop back for another pair of components

      end
* ---------------------------------------------------------------- SMC2PSAGMRot

      include '\forprogs\acc2vd.for'     
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
  

*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
*Write a file with az1, daz, (period, gmean, gmean-%, gmean_50%, gmean+%), repeated for a set of periods specficied in input  


      
