* ----------------------------------------------- SMC2Instr_Response
      Program SMC2Instr_Response

* Runs an acceleration file through an instrument response

*!Control file for program SMC2Instr_Response
*! Revision of program involving a change in the control file on this date:
*   07/19/10
*!Name of summary file:
* smc2instr_response.sum
*! instr_type = 'WA', 'WWSSN-SP', 'USGS-SP', 'PIDC-SP', ...
*  wwssn-sp
*!taper_beginning, taper_end
* 1.0 1.0
*!Write pad-stripped time series (Y/N)?
* Y
*!Character string to append to name of input file for output
* .wwssn-sp.smc
*!List of files 
* t0781pea.smc
*Stop
 
* Dates: 01/19/10 - Written by D. M. Boore, patterned after SMC2Jerk
!        07/19/10 - A slight revision, allowing for other instrument types
 
c
c Dimension and declaration statements:

      character precision*8
      character version_ctl*8, version_in*30
      character instr_type*20


      real a(:), yout(:) 
      allocatable :: a, yout

      integer npts, nc_f_a 
      real tend

      character date_c*8, time_c*10, ext4out*50, pad_stripped*10

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_sum*40, f_a*70, f_out*100

      character f_ctl*40
      logical f_ctl_exist, f_a_exist, rmv_trnd, use_v0_d0


      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2Instr_Response.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'SMC2Instr_Response.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

*Check version of control file
      version_ctl = ' '
      version_ctl = '07/19/10'
      call trim_c(version_ctl,nc_version_ctl)

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      if (version_ctl(1:nc_version_ctl) /=
     :    version_in(1:nc_version_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2Instr_Response:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      instr_type = ' '
      read(nu_ctl,'(a)') instr_type
      call trim_c(instr_type,nc_instr_type)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip) 
      read(nu_ctl,*) taper_beginning, taper_end
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      pad_stripped = ' '
      read(nu_ctl,'(a)') pad_stripped
      call upstr(pad_stripped)
      call trim_c(pad_stripped,nc_pad_stripped)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      ext4out = ' '
      read(nu_ctl,'(a)') ext4out
      call trim_c(ext4out,nc_ext4out)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      read_files: DO
      
        buf = ' '
        read(nu_ctl,'(a)',iostat = istatus) buf
        if (istatus /= 0) EXIT
      
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') CYCLE
        if(buf_upper(1:4) .eq. 'STOP') EXIT
      
        f_a = ' '
        f_a = buf(1:nc_buf)
        nc_f_a = nc_buf     
        inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
 
        if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE
        end if

* Passed tests; proceed

 
        call smc_npts(f_a(1:nc_f_a), npts, sps)

 
        allocate( a(npts+5), yout(2*npts) )

* Now read the data from the files:

        write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

        tskip = 0.0
        tlength = -1.0

        call SMCRead(f_a(1:nc_f_a), tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)

        precision = ' '
        if (int_head(47) .eq. 8) then
          precision = 'HIGH'
        else
          precision = 'STANDARD'
        end if
        
        rnull = real_head(1)
   
        sps = real_head(2)
        ncomments_a = int_head(16)
        npts = int_head(17)
        dt = 1.0/sps
        tend = float(npts-1)*dt

* Compute instrument response:

        call acc2seismo_response(a, npts, dt, instr_type, 
     :           taper_beginning, taper_end,
     :           pad_stripped(1:1),
     :           yout, nout)
 
* Write output:

        f_out = ' ' 
        f_out = f_a(1:nc_f_a)//ext4out(1:nc_ext4out)
        call trim_c(f_out, nc_f_out)

* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
        call DATE_AND_TIME( date_c, time_c )
* Date is returned as 'CCYYMMDD' (character date*8) 
* Time is returned as 'hhmmss.sss' (character time*10)

        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| '//instr_type(1:nc_instr_type)//
     :            ' response, from processing acc file '//f_a(1:nc_f_a)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| using program SMC2Instr_Response'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   on '//
     :    date_c(1:4)//'/'//date_c(5:6)//'/'//date_c(7:8)//
     :   ' at time '//time_c(1:2)//':'//time_c(3:4)//':'//time_c(5:10)

  
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments 
        itype = 0
        int_head(17) = nout


        call SMCWrite(f_out, precision,
     :                yout, char_head, itype, 
     :                    int_head, real_head, comments)

  
        deallocate (a, yout)

      END DO read_files

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ----------------------------------------------- SMC2Instr_Response


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\acc2seismo_response.for'
      include '\forprogs\smcpadf.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\fork.for'
      include '\forprogs\poles_zeros_response.for'
      include '\forprogs\poles_zeros_values.for'
      
      include '\forprogs\get_avg.for'
 