! ------------------------------- smc_periods_from_zero_crossings_and_extrema
      Program smc_periods_from_zero_crossings_and_extrema


* Computes the periods from zero crossings and the extrema (as used in magnitude
! calculations, not the many local extrema approximated in random vibration theory
! by the ratio of the 4th and 2nd spectral moments).
 
* Input from a control file; here is an example:
*!Control file for program smc_periods_from_zero_crossings_and_extrema
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following lines.
*! Revision of program involving a change in the control file on this date:
*   03/06/12
*!Name of summary file:
* smc_periods_from_zero_crossings_and_extrema.sum
*!Name of output file:
* smc_periods_from_zero_crossings_and_extrema.out
*!mean info:
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*! Note that if working with a processed record that includes the 
*! complete time series produced by the processing (e.g., the padded time series
*! for acausal filtering or the post-processing baseline corrected time series
*! produced from pad-stripped or pass-through data in the first NGA project), 
*! it is inadvisible to remove the mean as this may 
*! produce unintended distortions at long periods; in this case set t4mean < 0.0.
*! If the record is pad-stripped, however, as provided by most
*! data centers, without post-processing to remove drifts in displacement,
*! a mean determined from the overall record probably should be 
*! removed.  But in this case the displacement will probably be inaccurate
*! (compared to the displacement derived from the padded, filtered acceleration
*! time series), it may be that a better approach is to specify the initial
*! velocity and displacement of the record---I am working on that (11/26/10):
* 0  1.0 20.0  -999.
*!eps for zero crossing (measure period from zero crossings for portion of record
*! between first and last occurrences of abs(seism)>= eps*absmax(seism))
*  0.05
*!List of files 
* t0781pea.smc
* t0781ped.smc
*Stop
 
* Dates: 03/06/12 - Written by D.M. Boore, patterned after smc2vd and \smsim\mb_sim\mrs_mb_sim_td_drvr

c
c Dimension and declaration statements:

      real a(:) 
      allocatable :: a 
      
      integer npts, nc_f_a
      real tend
       
      integer status

      character date_c*8, time_c*11, extension_c*30

      character date_ctl_correct*8, date_ctl_in*30

      character char_head(11)*80, cmnts2skip(50)*80
      character comments(500)*80 
!      character comments(500)*200 

      real real_head(50)
      integer int_head(48)

      character buf*200, buf_upper*4, 
     :        f_sum*200, f_a*200, f_out*200

      character f_ctl*200, precision*20
      logical f_exist, f_a_exist, rmv_trnd, 
     :  use_v0_d0, v0_d0_from_smc_header

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :   '("Enter"=smc_periods_from_zero_crossings_and_extrema.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') 
     :    f_ctl = 'smc_periods_from_zero_crossings_and_extrema.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '03/06/12'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /=
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_sum,'(a)') ' Output of program '//
     :      'SMC_PERIODS_FROM_ZERO_CROSSINGS_AND_EXTREMA:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)
      write(nu_out,'(a)') ' Output of program '//
     :      'SMC_PERIODS_FROM_ZERO_CROSSINGS_AND_EXTREMA:'
      write(nu_out,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_out,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_out,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) eps4zc
      write(nu_sum,'(a)') 
     :  ' eps4zc:'
      write(nu_sum,*) 
     :    eps4zc
      write(nu_out,'(a)') 
     :  ' eps4zc:'
      write(nu_out,*) 
     :    eps4zc

      write(nu_out,'(1x,a70, 1x,a, 
     :               2x,a, 1x,a,
     :               1x,a, 1x,a,
     :                              )') 
     :  'f_smc', 'eps4zc', 
     :  'per_zc', 'per_extrema',
     :  'freq_zc', 'freq_extrema'
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
      tzc_avg = 0.0
      fzc_avg = 0.0
      
      loop over files: DO
        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if(status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf == 0 .or. buf(1:1) == '!') then
          CYCLE loop over files
        end if
        if(buf_upper(1:4) == 'STOP') then
          EXIT loop over files
        end if
      
        f_a = ' '
        f_a = buf(1:nc_buf)
        nc_f_a = nc_buf 
      
        f_a_exist = .false.
        inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
*      write(*,'(a)') ' f_a: ', f_a(1:nc_f_a)
*      write(*,'(a, 1x, l1)') ' f_a_exist = ', f_a_exist

        if (.not. f_a_exist) then
            write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
            CYCLE loop over files
        end if

* Passed tests; proceed

        num = num + 1

* Get number of points on this loop:

        call smc_npts(f_a(1:nc_f_a), npts, sps)

*      write(nu_sum,*)
*      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

        allocate( a(npts) )

* Now read the data from the files:

        write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

        tskip = 0.0
        tlength = -1.0
  
        call SMCRead(f_a(1:nc_f_a),  
     :               tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)
     
        precision = ' '
        if (int_head(47) == 8) then
          precision = 'HIGH'
        else
          precision = 'STANDARD'
        end if
        
        call trim_c(precision,nc_precision)
       
   
        rnull = real_head(1)
        sps = real_head(2)
        ncomments_a = int_head(16)
        npts = int_head(17)
        dt = 1.0/sps
        tend = float(npts-1)*dt
  
*      write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
*     :     'signal_in: dt=', dt, 
*     :     ' npts=', npts,' tend=', tend

* Remove a mean from the whole record

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
*  Determine the appropriate mean:
        if (itype_mean == 0) then
          n_t4mean = 0
          avg = 0.0
        else if (itype_mean == -1) then   ! use specified mean
          avg = spec_mean
        else if (itype_mean == 1) then
          if (t4mean_strt < 0.0) t4mean_strt = 0.0
        if (t4mean_stop > tend) t4mean_stop = tend
          n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
          n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
          n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
          t4mean = float(n_t4mean -1)/sps
*        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
          cum_avg = 0.0
          do i = n_t4mean_strt, n_t4mean_stop
            cum_avg = cum_avg + dble(a(i))
          end do
          avg = sngl(cum_avg/dble(real(n_t4mean)))
        else            ! shouldn't be here!
          write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
          write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
          deallocate (a)
          EXIT loop over files
        end if

! Remove the mean from the whole record:
        do i = 1, npts
          a(i) = a(i) - avg
        end do

! Obtain period from zero crossings:

        call period_from_zero_crossings(
     :        a, npts, dt, eps4zc, nstart_zc, nstop_zc, period_zc)

        tzc_avg = tzc_avg + period_zc
        fzc_avg = fzc_avg + 1.0/period_zc

! Obtain period of the extrema (this is the period used in magnitude calculations:

        call peak_trough_max_amp_period(a, npts, dt,
     :         amp_extrema, period_extrema, index_peak, time_peak)
 
! Write output:
     
        nc_f_a_for_output = min(nc_f_a, 70)
        write(nu_out,'(1x,a70, 3x,f4.2, 
     :     1x,f7.3, 5x,f7.3,
     :     1x,f7.3, 4x,f7.3
     :                                       )') 
     :     f_a(1:nc_f_a_for_output), eps4zc,
     :     period_zc, period_extrema,
     :     1.0/period_zc, 1.0/period_extrema
 
        deallocate (a)

      END DO loop over files

      tzc_avg = tzc_avg/real(num)
      fzc_avg = fzc_avg/real(num)
      
      write(nu_sum,*)
      write(nu_sum,'(1x,a, 1x,i3, 
     :                 1x,f7.3, 1x, f7.3)') 
     :  ' nfiles, average period_zc, freq_zc = ', num, 
     :                      tzc_avg, fzc_avg

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)
      close(nu_out)

      stop
      end
! ------------------------------- smc_periods_from_zero_crossings_and_extrema


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\distaz.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\datetime.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\period_from_zero_crossings.for'
      include '\forprogs\peak_trough_max_amp_period.for'
      include '\forprogs\get_extrema.for'
       


   
 